from django.utils.functional import cached_property

from . import factories
from .. import models


class UserFixture(object):

    @cached_property
    def staff(self):
        return factories.UserFactory(is_staff=True)

    @cached_property
    def user(self):
        return factories.UserFactory()

    @cached_property
    def global_support(self):
        return factories.UserFactory(is_support=True)


class CustomerFixture(UserFixture):

    @cached_property
    def customer(self):
        return factories.CustomerFactory()

    @cached_property
    def owner(self):
        owner = factories.UserFactory()
        self.customer.add_user(owner, models.CustomerRole.OWNER)
        return owner

    @cached_property
    def customer_support(self):
        support = factories.UserFactory()
        self.customer.add_user(support, models.CustomerRole.SUPPORT)
        return support


class ProjectFixture(CustomerFixture):

    @cached_property
    def project(self):
        return factories.ProjectFactory(customer=self.customer)

    @cached_property
    def admin(self):
        admin = factories.UserFactory()
        self.project.add_user(admin, models.ProjectRole.ADMINISTRATOR)
        return admin

    @cached_property
    def manager(self):
        manager = factories.UserFactory()
        self.project.add_user(manager, models.ProjectRole.MANAGER)
        return manager

    @cached_property
    def project_support(self):
        support = factories.UserFactory()
        self.project.add_user(support, models.ProjectRole.SUPPORT)
        return support


class ServiceFixture(ProjectFixture):

    @cached_property
    def service_settings(self):
        return factories.ServiceSettingsFactory(customer=self.customer)

    @cached_property
    def service(self):
        return factories.TestServiceFactory(settings=self.service_settings, customer=self.customer)

    @cached_property
    def service_project_link(self):
        return factories.TestServiceProjectLinkFactory(service=self.service, project=self.project)

    @cached_property
    def resource(self):
        return factories.TestNewInstanceFactory(service_project_link=self.service_project_link)



# -*- coding: utf-8 -*-
# Generated by Django 1.9.12 on 2017-04-07 17:31
from __future__ import unicode_literals

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('structure', '0046_shared_service_settings_customer'),
    ]

    operations = [
        migrations.CreateModel(
            name='PrivateServiceSettings',
            fields=[
            ],
            options={
                'proxy': True,
                'verbose_name_plural': 'Private provider settings',
            },
            bases=('structure.servicesettings',),
        ),
        migrations.CreateModel(
            name='SharedServiceSettings',
            fields=[
            ],
            options={
                'proxy': True,
                'verbose_name_plural': 'Shared provider settings',
            },
            bases=('structure.servicesettings',),
        ),
    ]



from __future__ import unicode_literals

from collections import defaultdict
import json

import six
from django import forms
from django.conf import settings
from django.conf.urls import url
from django.contrib import admin, messages
from django.contrib.admin import forms as admin_forms
from django.contrib.auth import admin as auth_admin, get_user_model
from django.contrib.auth.models import Group
from django.core.exceptions import ValidationError
from django.urls import reverse
from django.utils.html import format_html_join
from django.utils.safestring import mark_safe
from django.utils.translation import ugettext_lazy as _
from rest_framework import permissions as rf_permissions
from reversion.admin import VersionAdmin

from waldur_core.core import models
from waldur_core.core.authentication import can_access_admin_site


def get_admin_url(obj):
    return reverse('admin:%s_%s_change' % (obj._meta.app_label, obj._meta.model_name), args=[obj.id])


def render_to_readonly(value):
    return "<p>{0}</p>".format(value)


class ReadonlyTextWidget(forms.TextInput):
    def format_value(self, value):
        return value

    def render(self, name, value, attrs=None, renderer=None):
        return render_to_readonly(self.format_value(value))


class PasswordWidget(forms.PasswordInput):
    template_name = 'admin/core/widgets/password-widget.html'

    def __init__(self, attrs=None):
        super(PasswordWidget, self).__init__(attrs, render_value=True)


def format_json_field(value):
    template = '<div><pre style="overflow: hidden">{0}</pre></div>'
    formatted_value = json.dumps(value, indent=True)
    return template.format(formatted_value)


class OptionalChoiceField(forms.ChoiceField):
    def __init__(self, choices=(), *args, **kwargs):
        empty = [('', '---------')]
        choices = empty + sorted(choices, key=lambda (code, label): label)
        super(OptionalChoiceField, self).__init__(choices, *args, **kwargs)


class UserCreationForm(auth_admin.UserCreationForm):
    class Meta(object):
        model = get_user_model()
        fields = ("username",)

    # overwritten to support custom User model
    def clean_username(self):
        # Since User.username is unique, this check is redundant,
        # but it sets a nicer error message than the ORM. See #13147.
        username = self.cleaned_data["username"]
        try:
            get_user_model()._default_manager.get(username=username)
        except get_user_model().DoesNotExist:
            return username
        raise forms.ValidationError(
            self.error_messages['duplicate_username'],
            code='duplicate_username',
        )


class UserChangeForm(auth_admin.UserChangeForm):
    class Meta(object):
        model = get_user_model()
        fields = '__all__'

    def __init__(self, *args, **kwargs):
        super(UserChangeForm, self).__init__(*args, **kwargs)
        competences = [(key, key) for key in settings.WALDUR_CORE.get('USER_COMPETENCE_LIST', [])]
        self.fields['preferred_language'] = OptionalChoiceField(choices=settings.LANGUAGES, required=False)
        self.fields['competence'] = OptionalChoiceField(choices=competences, required=False)

    def clean_civil_number(self):
        # Empty string should be converted to None.
        # Otherwise uniqueness constraint is violated.
        # See also: http://stackoverflow.com/a/1400046/175349
        civil_number = self.cleaned_data.get('civil_number')
        if civil_number:
            return civil_number.strip()
        return None


class UserAdmin(auth_admin.UserAdmin):
    list_display = ('username', 'uuid', 'email', 'full_name', 'native_name', 'is_active', 'is_staff', 'is_support')
    search_fields = ('username', 'uuid', 'full_name', 'native_name', 'email', 'civil_number')
    list_filter = ('is_active', 'is_staff', 'is_support')
    fieldsets = (
        (None, {'fields': ('username', 'password', 'registration_method', 'uuid')}),
        (_('Personal info'), {'fields': (
            'civil_number', 'full_name', 'native_name', 'email',
            'preferred_language', 'competence', 'phone_number'
        )}),
        (_('Organization'), {'fields': ('organization', 'organization_approved', 'job_title')}),
        (_('Permissions'), {'fields': ('is_active', 'is_staff', 'is_support', 'customer_roles', 'project_roles')}),
        (_('Important dates'), {'fields': ('last_login', 'date_joined', 'agreement_date')}),
    )
    readonly_fields = ('registration_method', 'agreement_date', 'customer_roles', 'project_roles', 'uuid')
    form = UserChangeForm
    add_form = UserCreationForm

    def customer_roles(self, instance):
        from waldur_core.structure.models import CustomerPermission
        permissions = CustomerPermission.objects.filter(user=instance, is_active=True).order_by('customer')

        return format_html_join(
            mark_safe('<br/>'),  # nosec
            '<a href={}>{}</a>',
            ((get_admin_url(permission.customer), six.text_type(permission)) for permission in permissions),
        ) or mark_safe("<span class='errors'>%s</span>" % _('User has no roles in any organization.'))  # nosec

    customer_roles.short_description = _('Roles in organizations')

    def project_roles(self, instance):
        from waldur_core.structure.models import ProjectPermission
        permissions = ProjectPermission.objects.filter(user=instance, is_active=True).order_by('project')

        return format_html_join(
            mark_safe('<br/>'),  # nosec
            '<a href={}>{}</a>',
            ((get_admin_url(permission.project), six.text_type(permission)) for permission in permissions),
        ) or mark_safe("<span class='errors'>%s</span>" % _('User has no roles in any project.'))  # nosec

    project_roles.short_description = _('Roles in projects')


class SshPublicKeyAdmin(admin.ModelAdmin):
    list_display = ('user', 'name', 'fingerprint')
    search_fields = ('user', 'name', 'fingerprint')
    readonly_fields = ('user', 'name', 'fingerprint', 'public_key')


class CustomAdminAuthenticationForm(admin_forms.AdminAuthenticationForm):
    error_messages = {
        'invalid_login': _("Please enter the correct %(username)s and password "
                           "for a staff or a support account. Note that both fields may be "
                           "case-sensitive."),
    }

    def confirm_login_allowed(self, user):
        if not can_access_admin_site(user):
            return super(CustomAdminAuthenticationForm, self).confirm_login_allowed(user)


class CustomAdminSite(admin.AdminSite):
    site_title = _('Waldur MasterMind admin')
    site_header = _('Waldur MasterMind administration')
    index_title = _('Waldur MasterMind administration')
    login_form = CustomAdminAuthenticationForm

    def has_permission(self, request):
        is_safe = request.method in rf_permissions.SAFE_METHODS
        return can_access_admin_site(request.user) and (is_safe or request.user.is_staff)

    @classmethod
    def clone_default(cls):
        instance = cls()
        instance._registry = admin.site._registry.copy()
        instance._actions = admin.site._actions.copy()
        instance._global_actions = admin.site._global_actions.copy()
        return instance

admin_site = CustomAdminSite.clone_default()
admin.site = admin_site
admin.site.register(models.User, UserAdmin)
admin.site.register(models.SshPublicKey, SshPublicKeyAdmin)

# TODO: Extract common classes to admin_utils module and remove hack.
# This hack is needed because admin is imported several times.
# Please note that admin module should NOT be imported by other apps.
if admin.site.is_registered(Group):
    admin.site.unregister(Group)


class ReversionAdmin(VersionAdmin):
    def add_view(self, request, form_url='', extra_context=None):
        # Revision creation is ignored in this method because it has to be implemented in model.save method
        return super(VersionAdmin, self).add_view(request, form_url, extra_context)

    def change_view(self, request, object_id, form_url='', extra_context=None):
        # Revision creation is ignored in this method because it has to be implemented in model.save method
        return super(VersionAdmin, self).change_view(request, object_id, form_url, extra_context)


class ExecutorAdminAction(object):
    """ Add executor as action to admin model.

    Usage example:
        class PullSecurityGroups(ExecutorAdminAction):
            executor = executors.TenantPullSecurityGroupsExecutor  # define executor
            short_description = 'Pull security groups'  # description for admin page

            def validate(self, tenant):
                if tenant.state != Tenant.States.OK:
                    raise ValidationError('Tenant has to be in state OK to pull security groups.')

        pull_security_groups = PullSecurityGroups()  # this action could be registered as admin action

    """
    executor = NotImplemented

    def __call__(self, admin_class, request, queryset):
        errors = defaultdict(list)
        successfully_executed = []
        for instance in queryset:
            try:
                self.validate(instance)
            except ValidationError as e:
                errors[six.text_type(e)].append(instance)
            else:
                self.executor.execute(instance)
                successfully_executed.append(instance)

        if successfully_executed:
            message = _('Operation was successfully scheduled for %(count)d instances: %(names)s') % dict(
                count=len(successfully_executed),
                names=', '.join([six.text_type(i) for i in successfully_executed])
            )
            admin_class.message_user(request, message)

        for error, instances in errors.items():
            message = _('Failed to schedule operation for %(count)d instances: %(names)s. Error: %(message)s') % dict(
                count=len(instances),
                names=', '.join([six.text_type(i) for i in instances]),
                message=error,
            )
            admin_class.message_user(request, message, level=messages.ERROR)

    def validate(self, instance):
        """ Raise validation error if action cannot be performed for given instance """
        pass


class ExtraActionsMixin(object):
    """
    Allows to add extra actions to admin list page.
    """
    change_list_template = 'admin/core/change_list.html'

    def get_extra_actions(self):
        raise NotImplementedError('Method "get_extra_actions" should be implemented in ExtraActionsMixin.')

    def get_urls(self):
        """
        Inject extra action URLs.
        """
        urls = []

        for action in self.get_extra_actions():
            regex = r'^{}/$'.format(self._get_action_href(action))
            view = self.admin_site.admin_view(action)
            urls.append(url(regex, view))

        return urls + super(ExtraActionsMixin, self).get_urls()

    def changelist_view(self, request, extra_context=None):
        """
        Inject extra links into template context.
        """
        links = []

        for action in self.get_extra_actions():
            links.append({
                'label': self._get_action_label(action),
                'href': self._get_action_href(action)
            })

        extra_context = extra_context or {}
        extra_context['extra_links'] = links

        return super(ExtraActionsMixin, self).changelist_view(
            request, extra_context=extra_context,
        )

    def _get_action_href(self, action):
        return action.__name__

    def _get_action_label(self, action):
        return getattr(action, 'name', action.__name__.replace('_', ' ').capitalize())


class UpdateOnlyModelAdmin(object):

    def has_add_permission(self, request, obj=None):
        return False

    def has_delete_permission(self, request, obj=None):
        return False



import datetime

from django.contrib.contenttypes.models import ContentType
from django.test import TransactionTestCase
from freezegun import freeze_time

from waldur_core.cost_tracking import models, ConsumableItem
from waldur_core.cost_tracking.tests import factories
from waldur_core.structure.tests import factories as structure_factories


class ConsumptionDetailsTest(TransactionTestCase):

    def setUp(self):
        price_estimate = factories.PriceEstimateFactory(year=2016, month=8)
        self.consumption_details = factories.ConsumptionDetailsFactory(price_estimate=price_estimate)
        self.storage_item = ConsumableItem('storage', '1 MB')

    def test_current_configuration_update(self):
        """ Test that consumed_before_modification field stores consumed items on configuration update """
        # Resource used some consumables
        with freeze_time("2016-08-08 11:00:00"):
            old_configuration = {self.storage_item: 1024, ConsumableItem('ram', '1 MB'): 512}
            self.consumption_details.update_configuration(old_configuration)
        # After 2 hours resource configuration was updated
        with freeze_time("2016-08-08 13:00:00"):
            self.consumption_details.update_configuration({self.storage_item: 2048})
        # Details of consumed items should be stored
        HOURS_BEFORE_UPDATE = 2
        for consumable_item, usage in old_configuration.items():
            self.assertEqual(self.consumption_details.consumed_before_update[consumable_item],
                             old_configuration[consumable_item] * HOURS_BEFORE_UPDATE * 60)

    def test_cannot_update_configuration_for_previous_month(self):
        with freeze_time("2016-09-01"):
            self.assertRaises(
                models.ConsumptionDetailUpdateError,
                lambda: self.consumption_details.update_configuration({self.storage_item: 2048}))

    def test_consumed_in_month(self):
        """ Property "consumed_in_month" should return how much consumables resource will use for whole month """
        # Resource used some consumables
        start_time = datetime.datetime(2016, 8, 8, 11, 0)
        with freeze_time(start_time):
            old_configuration = {self.storage_item: 1024}
            self.consumption_details.update_configuration(old_configuration)
        # After 2 hours resource configuration was updated
        change_time = datetime.datetime(2016, 8, 8, 13, 0)
        with freeze_time(change_time):
            new_configuration = {self.storage_item: 2048}
            self.consumption_details.update_configuration(new_configuration)

        # Expected consumption should assume that resource will use current
        # configuration to the end of month and add 2 hours of old configuration
        month_end = datetime.datetime(2016, 8, 31, 23, 59, 59)
        expected = (int((change_time - start_time).total_seconds() / 60) * old_configuration[self.storage_item] +
                    int((month_end - change_time).total_seconds() / 60) * new_configuration[self.storage_item])
        self.assertEqual(self.consumption_details.consumed_in_month[self.storage_item], expected)

    def test_consumed_until_now(self):
        # Resource used some consumables
        start_time = datetime.datetime(2016, 8, 8, 11, 0)

        with freeze_time(start_time):
            old_configuration = {self.storage_item: 1024}
            self.consumption_details.update_configuration(old_configuration)
        # After two hours resource configuration was updated
        change_time = datetime.datetime(2016, 8, 8, 13, 0)
        with freeze_time(change_time):
            new_configuration = {self.storage_item: 2048}
            self.consumption_details.update_configuration(new_configuration)

        # After one more hour we are checking how much resources were consumed
        now_time = datetime.datetime(2016, 8, 8, 14, 0)
        with freeze_time(now_time):
            expected = (
                2 * 60 * old_configuration[self.storage_item] +  # Resource worked two hours with old configuration
                1 * 60 * new_configuration[self.storage_item])   # And one hour with new configuration
            self.assertEqual(self.consumption_details.consumed_until_now[self.storage_item], expected)


class PriceListItemTest(TransactionTestCase):

    def test_get_for_resource(self):
        resource = structure_factories.TestNewInstanceFactory()
        resource_content_type = ContentType.objects.get_for_model(resource)
        service = resource.service_project_link.service
        # resource has two default price list items
        default_item1 = models.DefaultPriceListItem.objects.create(
            resource_content_type=resource_content_type, item_type='flavor', key='small', value=10)
        default_item2 = models.DefaultPriceListItem.objects.create(
            resource_content_type=resource_content_type, item_type='storage', key='1 GB', value=0.5)
        # the second item is overridden be regular price list item
        item = models.PriceListItem.objects.create(default_price_list_item=default_item2, service=service)

        expected = {default_item1, item}
        self.assertSetEqual(models.PriceListItem.get_for_resource(resource), expected)


class DefaultPriceListItemTest(TransactionTestCase):

    def test_get_consumable_items_pretty_names(self):
        item_type, key = 'flavor', 'small'
        price_list_item = factories.DefaultPriceListItemFactory(item_type=item_type, key=key, name='Pretty name')
        consumable_item = ConsumableItem(item_type, key)

        expected = {consumable_item: price_list_item.name}
        actual = models.DefaultPriceListItem.get_consumable_items_pretty_names(
            price_list_item.resource_content_type, [consumable_item])
        self.assertDictEqual(actual, expected)



from waldur_core.structure import filters as structure_filters
from waldur_core.structure import views as structure_views

from . import models, serializers


class TestServiceViewSet(structure_views.BaseServiceViewSet):
    queryset = models.TestService.objects.all()
    serializer_class = serializers.ServiceSerializer


class TestServiceProjectLinkViewSet(structure_views.BaseServiceProjectLinkViewSet):
    queryset = models.TestServiceProjectLink.objects.all()
    serializer_class = serializers.ServiceProjectLinkSerializer


class TestNewInstanceFilter(structure_filters.BaseResourceFilter):
    class Meta(structure_filters.BaseResourceFilter.Meta):
        model = models.TestNewInstance


class TestNewInstanceViewSet(structure_views.ResourceViewSet):
    queryset = models.TestNewInstance.objects.all()
    serializer_class = serializers.NewInstanceSerializer
    filter_class = TestNewInstanceFilter

    def perform_create(self, serializer):
        return serializer.save()

    def perform_update(self, serializer):
        return serializer.save()



from ddt import ddt, data
from django.utils import timezone
from freezegun import freeze_time
from rest_framework import status

from waldur_core.core.tests.helpers import override_waldur_core_settings
from waldur_core.structure.tests import factories as structure_factories

from .. import models
from . import factories
from .base_test import BaseCostTrackingTest


@ddt
class PriceEstimateListTest(BaseCostTrackingTest):
    def setUp(self):
        super(PriceEstimateListTest, self).setUp()

        self.link_price_estimate = factories.PriceEstimateFactory(
            year=2012, month=10, scope=self.service_project_link)
        self.project_price_estimate = factories.PriceEstimateFactory(scope=self.project, year=2015, month=7)

    @data('owner', 'manager', 'administrator')
    def test_user_can_see_price_estimate_for_his_project(self, user):
        self.client.force_authenticate(self.users[user])
        response = self.client.get(factories.PriceEstimateFactory.get_list_url())

        self.assertEqual(response.status_code, status.HTTP_200_OK)
        self.assertIn(self.project_price_estimate.uuid.hex, [obj['uuid'] for obj in response.data])

    @data('owner', 'manager', 'administrator')
    def test_user_cannot_see_price_estimate_for_not_his_project(self, user):
        other_price_estimate = factories.PriceEstimateFactory()

        self.client.force_authenticate(self.users[user])
        response = self.client.get(factories.PriceEstimateFactory.get_list_url())

        self.assertEqual(response.status_code, status.HTTP_200_OK)
        self.assertNotIn(other_price_estimate.uuid.hex, [obj['uuid'] for obj in response.data])

    def test_user_can_filter_price_estimate_by_scope(self):
        self.client.force_authenticate(self.users['owner'])
        response = self.client.get(
            factories.PriceEstimateFactory.get_list_url(),
            data={'scope': structure_factories.ProjectFactory.get_url(self.project)})

        self.assertEqual(response.status_code, status.HTTP_200_OK)
        self.assertEqual(len(response.data), 1)
        self.assertEqual(response.data[0]['uuid'], self.project_price_estimate.uuid.hex)

    def test_user_can_filter_price_estimates_by_date(self):
        self.client.force_authenticate(self.users['administrator'])
        response = self.client.get(
            factories.PriceEstimateFactory.get_list_url(),
            data={'date': '{}.{}'.format(self.link_price_estimate.year, self.link_price_estimate.month)})

        self.assertEqual(response.status_code, status.HTTP_200_OK)
        self.assertEqual(len(response.data), 1)
        self.assertEqual(response.data[0]['uuid'], self.link_price_estimate.uuid.hex)

    def test_user_can_filter_price_estimates_by_date_range(self):
        self.client.force_authenticate(self.users['manager'])
        response = self.client.get(
            factories.PriceEstimateFactory.get_list_url(),
            data={'start': '{}.{}'.format(self.link_price_estimate.year, self.link_price_estimate.month + 1),
                  'end': '{}.{}'.format(self.project_price_estimate.year, self.project_price_estimate.month + 1)})

        self.assertEqual(response.status_code, status.HTTP_200_OK)
        self.assertEqual(len(response.data), 1)
        self.assertEqual(response.data[0]['uuid'], self.project_price_estimate.uuid.hex)

    def test_user_receive_error_on_filtering_by_not_visible_for_him_object(self):
        data = {'scope': structure_factories.ProjectFactory.get_url()}

        self.client.force_authenticate(self.users['administrator'])
        response = self.client.get(factories.PriceEstimateFactory.get_list_url(), data=data)

        self.assertEqual(response.status_code, status.HTTP_400_BAD_REQUEST)

    def test_user_can_define_children_visibility_depth(self):
        customer_price_estimate = factories.PriceEstimateFactory(scope=self.customer, year=2015, month=7)
        customer_price_estimate.children.add(self.project_price_estimate)
        spl_price_estimate = factories.PriceEstimateFactory(scope=self.service_project_link, year=2015, month=7)
        self.project_price_estimate.children.add(spl_price_estimate)

        self.client.force_authenticate(self.users['owner'])

        response = self.client.get(factories.PriceEstimateFactory.get_url(customer_price_estimate), data={'depth': 1})

        self.assertEqual(response.status_code, status.HTTP_200_OK)
        # with visibility depth 1 we want to see customer estimate children
        self.assertEqual(len(response.data['children']), 1)
        project_estimate_data = response.data['children'][0]
        self.assertEqual(project_estimate_data['uuid'], self.project_price_estimate.uuid.hex)
        # with visibility depth 1 we do not want to see grandchildren
        self.assertNotIn('children', project_estimate_data)


class PriceEstimateUpdateTest(BaseCostTrackingTest):
    def setUp(self):
        super(PriceEstimateUpdateTest, self).setUp()

        self.price_estimate = factories.PriceEstimateFactory(scope=self.service_project_link)
        self.valid_data = {
            'scope': structure_factories.TestServiceProjectLinkFactory.get_url(self.service_project_link),
            'total': 100,
            'details': {'ram': 50, 'disk': 50},
            'month': 7,
            'year': 2015,
        }

    def test_price_estimate_scope_cannot_be_updated(self):
        other_service_project_link = structure_factories.TestServiceProjectLinkFactory(project=self.project)
        self.valid_data['scope'] = structure_factories.TestServiceProjectLinkFactory.get_url(
            other_service_project_link)

        self.client.force_authenticate(self.users['staff'])
        response = self.client.patch(factories.PriceEstimateFactory.get_url(self.price_estimate), data=self.valid_data)

        self.assertEqual(response.status_code, status.HTTP_405_METHOD_NOT_ALLOWED)
        reread_price_estimate = models.PriceEstimate.objects.get(id=self.price_estimate.id)
        self.assertNotEqual(reread_price_estimate.scope, other_service_project_link)

    def test_autocalculated_estimate_cannot_be_manually_updated(self):
        self.client.force_authenticate(self.users['staff'])
        response = self.client.patch(factories.PriceEstimateFactory.get_url(self.price_estimate), data=self.valid_data)

        self.assertEqual(response.status_code, status.HTTP_405_METHOD_NOT_ALLOWED)


class PriceEstimateDeleteTest(BaseCostTrackingTest):
    def setUp(self):
        super(PriceEstimateDeleteTest, self).setUp()
        self.project_price_estimate = factories.PriceEstimateFactory(scope=self.project)

    def test_autocreated_price_estimate_cannot_be_deleted(self):
        self.client.force_authenticate(self.users['staff'])
        response = self.client.delete(factories.PriceEstimateFactory.get_url(self.project_price_estimate))

        self.assertEqual(response.status_code, status.HTTP_405_METHOD_NOT_ALLOWED)


class ScopeTypeFilterTest(BaseCostTrackingTest):
    def setUp(self):
        super(ScopeTypeFilterTest, self).setUp()
        resource = structure_factories.TestNewInstanceFactory(service_project_link=self.service_project_link)
        self.estimates = {
            'customer': models.PriceEstimate.objects.get(scope=self.customer),
            'service': models.PriceEstimate.objects.get(scope=self.service),
            'project': models.PriceEstimate.objects.get(scope=self.project),
            'service_project_link': models.PriceEstimate.objects.get(scope=self.service_project_link),
            'resource': models.PriceEstimate.objects.get(scope=resource),
        }

    def test_user_can_filter_price_estimate_by_scope_type(self):
        self.client.force_authenticate(self.users['owner'])
        for scope_type, estimate in self.estimates.items():
            response = self.client.get(
                factories.PriceEstimateFactory.get_list_url(),
                data={'scope_type': scope_type})

            self.assertEqual(response.status_code, status.HTTP_200_OK)
            self.assertEqual(len(response.data), 1, response.data)
            self.assertEqual(response.data[0]['uuid'], estimate.uuid.hex)


class CustomerFilterTest(BaseCostTrackingTest):
    def setUp(self):
        super(CustomerFilterTest, self).setUp()
        resource = structure_factories.TestNewInstanceFactory()
        link = resource.service_project_link
        customer = link.customer
        project = link.project
        service = link.service

        scopes = {link, customer, project, service, resource}
        self.estimates = {models.PriceEstimate.objects.get(scope=scope) for scope in scopes}
        self.customer = customer

        resource2 = structure_factories.TestNewInstanceFactory()
        resource2_estimate = factories.PriceEstimateFactory(scope=resource2)
        resource2_estimate.create_ancestors()

    def test_user_can_filter_price_estimate_by_customer_uuid(self):
        self.client.force_authenticate(self.users['staff'])
        response = self.client.get(
            factories.PriceEstimateFactory.get_list_url(),
            data={'customer': self.customer.uuid.hex})

        self.assertEqual(response.status_code, status.HTTP_200_OK)
        self.assertEqual({estimate['uuid'] for estimate in response.data},
                         {estimate.uuid.hex for estimate in self.estimates})



from operator import or_

from django.db import models

from waldur_core.core.managers import GenericKeyMixin, SummaryQuerySet


def filter_queryset_for_user(queryset, user):
    filtered_relations = ('customer', 'project')

    if user is None or user.is_staff or user.is_support:
        return queryset

    def create_q(entity):
        try:
            path = getattr(permissions, '%s_path' % entity)
        except AttributeError:
            return None

        role = getattr(permissions, '%s_role' % entity, None)

        if path == 'self':
            prefix = ''
        else:
            prefix = path + '__'

        kwargs = {
            prefix + 'permissions__user': user,
            prefix + 'permissions__is_active': True
        }

        if role:
            kwargs[prefix + 'permissions__role'] = role

        return models.Q(**kwargs)

    try:
        permissions = queryset.model.Permissions
    except AttributeError:
        return queryset

    q_objects = [q_object for q_object in (
        create_q(entity) for entity in filtered_relations
    ) if q_object is not None]

    try:
        # Add extra query which basically allows to
        # additionally filter by some flag and ignore permissions
        extra_q = getattr(permissions, 'extra_query')
    except AttributeError:
        pass
    else:
        q_objects.append(models.Q(**extra_q))

    try:
        # Whether both customer and project filtering requested?
        any_of_q = reduce(or_, q_objects)
        return queryset.filter(any_of_q).distinct()
    except TypeError:
        # Or any of customer and project filtering requested?
        return queryset.filter(q_objects[0])
    except IndexError:
        # Looks like no filters are there
        return queryset


class StructureQueryset(models.QuerySet):
    """ Provides additional filtering by customer or project (based on permission definition).

        Example:

            .. code-block:: python

                Instance.objects.filter(project=12)

                Droplet.objects.filter(
                    customer__name__startswith='A',
                    state=Droplet.States.ONLINE)

                Droplet.objects.filter(Q(customer__name='Alice') | Q(customer__name='Bob'))
    """

    def exclude(self, *args, **kwargs):
        return super(StructureQueryset, self).exclude(
            *[self._patch_query_argument(a) for a in args],
            **self._filter_by_custom_fields(**kwargs))

    def filter(self, *args, **kwargs):
        return super(StructureQueryset, self).filter(
            *[self._patch_query_argument(a) for a in args],
            **self._filter_by_custom_fields(**kwargs))

    def _patch_query_argument(self, arg):
        # patch Q() objects if passed and add support of custom fields
        if isinstance(arg, models.Q):
            children = []
            for opt in arg.children:
                if isinstance(opt, models.Q):
                    children.append(self._patch_query_argument(opt))
                else:
                    args = self._filter_by_custom_fields(**dict([opt]))
                    children.append(tuple(args.items())[0])
            arg.children = children
        return arg

    def _filter_by_custom_fields(self, **kwargs):
        # traverse over filter arguments in search of custom fields
        args = {}
        fields = [f.name for f in self.model._meta.get_fields()]
        for field, val in kwargs.items():
            base_field = field.split('__')[0]
            if base_field in fields:
                args.update(**{field: val})
            elif base_field in ('customer', 'project'):
                args.update(self._filter_by_permission_fields(base_field, field, val))
            else:
                args.update(**{field: val})

        return args

    def _filter_by_permission_fields(self, name, field, value):
        # handle fields connected via permissions relations
        extra = '__'.join(field.split('__')[1:]) if '__' in field else None
        try:
            # look for the target field path in Permissions class,
            path = getattr(self.model.Permissions, '%s_path' % name)
        except AttributeError:
            # fallback to FieldError if it's missed
            return {field: value}
        else:
            if path == 'self':
                if extra:
                    return {extra: value}
                else:
                    return {'pk': value.pk if isinstance(value, models.Model) else value}
            else:
                if extra:
                    path += '__' + extra
                return {path: value}


StructureManager = models.Manager.from_queryset(StructureQueryset)


class ResourceSummaryQuerySet(SummaryQuerySet):
    # Hack for permissions
    @property
    def model(self):
        from waldur_core.structure.models import ResourceMixin
        return ResourceMixin


class ServiceSummaryQuerySet(SummaryQuerySet):
    # Hack for permissions
    @property
    def model(self):
        from waldur_core.structure.models import Service
        return Service


class ServiceSettingsManager(GenericKeyMixin, models.Manager):
    """ Allows to filter and get service settings by generic key """

    def get_available_models(self):
        """ Return list of models that are acceptable """
        from waldur_core.structure.models import ResourceMixin
        return ResourceMixin.get_all_models()


class SharedServiceSettingsManager(ServiceSettingsManager):

    def get_queryset(self):
        return super(SharedServiceSettingsManager, self).get_queryset().filter(shared=True)


class PrivateServiceSettingsManager(ServiceSettingsManager):

    def get_queryset(self):
        return super(PrivateServiceSettingsManager, self).get_queryset().filter(shared=False)



from __future__ import unicode_literals

import logging

from django.conf import settings
from elasticsearch import Elasticsearch

from waldur_core.core.utils import datetime_to_timestamp


logger = logging.getLogger(__name__)


class ElasticsearchError(Exception):
    pass


class ElasticsearchClientError(ElasticsearchError):
    pass


class ElasticsearchResultListError(ElasticsearchError):
    pass


class EmptyQueryset(object):
    def __len__(self):
        return 0

    def count(self):
        return 0

    def __getitem__(self, key):
        return []


class ElasticsearchResultList(object):
    """ List of results acceptable by django pagination """

    def __init__(self):
        self.client = ElasticsearchClient()

    def filter(self, should_terms=None, must_terms=None, must_not_terms=None, search_text='', start=None, end=None):
        setattr(self, 'total', None)
        self.client.prepare_search_body(
            should_terms=should_terms,
            must_terms=must_terms,
            must_not_terms=must_not_terms,
            search_text=search_text,
            start=start,
            end=end,
        )
        return self

    def order_by(self, sort):
        self.sort = sort
        return self

    def count(self):
        return self.client.get_count()

    def aggregated_count(self, ranges):
        return self.client.get_aggregated_by_timestamp_count(ranges)

    def _get_events(self, from_, size):
        return self.client.get_events(
            from_=from_,
            size=size,
            sort=getattr(self, 'sort', '-@timestamp'),
        )

    def __len__(self):
        if not hasattr(self, 'total') or self.total is None:
            self.total = self._get_events(0, 1)['total']
        return self.total

    def __getitem__(self, key):
        if isinstance(key, slice):
            if key.step is not None and key.step != 1:
                raise ElasticsearchResultListError('ElasticsearchResultList can be iterated only with step 1')
            start = key.start if key.start is not None else 0
            events_and_total = self._get_events(start, key.stop - start)
        else:
            events_and_total = self._get_events(key, 1)
        self.total = events_and_total['total']
        return events_and_total['events']


def _execute_if_not_empty(func):
    """ Execute function only if one of input parameters is not empty """
    def wrapper(*args, **kwargs):
        if any(args[1:]) or any(kwargs.items()):
            return func(*args, **kwargs)
    return wrapper


class ElasticsearchClient(object):

    class SearchBody(dict):
        FTS_FIELDS = (
            'message', 'customer_abbreviation', 'importance', 'project_group_name',
            'project_name', 'user_native_name', 'user_full_name')

        def __init__(self):
            self.queries = {}
            self.timestamp_filter = {}
            self.should_terms_filter = {}
            self.must_terms_filter = {}
            self.must_not_terms_filter = {}
            self.timestamp_ranges = []

        @_execute_if_not_empty
        def set_should_terms(self, terms):
            self.should_terms_filter.update({key: map(str, value) for key, value in terms.items()})

        @_execute_if_not_empty
        def set_must_terms(self, terms):
            self.must_terms_filter.update({key: map(str, value) for key, value in terms.items()})

        @_execute_if_not_empty
        def set_must_not_terms(self, terms):
            self.must_not_terms_filter.update({key: map(str, value) for key, value in terms.items()})

        @_execute_if_not_empty
        def set_search_text(self, search_text):
            self.queries['search_text'] = ' OR '.join(
                [self._format_to_elasticsearch_field_filter(field, [search_text]) for field in self.FTS_FIELDS])

        @_execute_if_not_empty
        def set_timestamp_filter(self, start=None, end=None):
            if start is not None:
                self.timestamp_filter['gte'] = start.strftime('%Y-%m-%dT%H:%M:%S')
            if end is not None:
                self.timestamp_filter['lt'] = end.strftime('%Y-%m-%dT%H:%M:%S')

        @_execute_if_not_empty
        def set_timestamp_ranges(self, ranges):
            self.timestamp_ranges = []
            for r in ranges:
                timestamp_range = {}
                if 'start' in r:
                    timestamp_range['from'] = self.datetime_to_elasticsearch_timestamp(r['start'])
                if 'end' in r:
                    timestamp_range['to'] = self.datetime_to_elasticsearch_timestamp(r['end'])
                self.timestamp_ranges.append(timestamp_range)

        def prepare(self):
            # Valid event has event_type field
            self['query'] = {
                'bool': {
                    'must': [
                        {
                            'exists': {
                                'field': 'event_type'
                            }
                        }
                    ]
                }
            }

            if self.queries:
                self['query']['bool']['filter'] = {
                    'query_string': {
                        'query': ' AND '.join('(' + search_query + ')' for search_query in self.queries.values())
                    }
                }

            if self.should_terms_filter:
                self['query']['bool']['should'] = [
                    {'terms': {key: value}} for key, value in self.should_terms_filter.items()
                ]

            if self.must_terms_filter:
                self['query']['bool']['must'].extend([
                    {'terms': {key: value}} for key, value in self.must_terms_filter.items()
                ])

            if self.must_not_terms_filter:
                self['query']['bool']['must_not'] = [
                    {'terms': {key: value}} for key, value in self.must_not_terms_filter.items()
                ]

            if self.timestamp_filter:
                self['query']['bool']['must'].append({
                    'range': {'@timestamp': self.timestamp_filter}})

            if self.timestamp_ranges:
                self['aggs'] = {
                    'timestamp_ranges': {
                        'date_range': {
                            'field': '@timestamp',
                            'ranges': self.timestamp_ranges,
                        },
                    }
                }

        def datetime_to_elasticsearch_timestamp(self, dt):
            """ Elasticsearch calculates timestamp in milliseconds """
            return datetime_to_timestamp(dt) * 1000

        def _escape_elasticsearch_field_value(self, field_value):
            """
            Remove double quotes from field value

            Elasticsearch receives string query where all user input is strings in double quotes.
            But if input itself contains double quotes - elastic treat them as end of string, so we have to remove double
            quotes from search string.
            """
            return field_value.replace('\"', '')

        def _format_to_elasticsearch_field_filter(self, field_name, field_values):
            """
            Return string '<field_name>:("<field_value1>", "<field_value2>"...)'
            """
            excaped_field_values = [self._escape_elasticsearch_field_value(value) for value in field_values]
            return '%s:("%s")' % (field_name, '", "'.join(excaped_field_values))

    def __init__(self):
        self.client = self._get_client()

    def prepare_search_body(self, should_terms=None, must_terms=None, must_not_terms=None, search_text='', start=None, end=None):
        """
        Prepare body for elasticsearch query

        Search parameters
        ^^^^^^^^^^^^^^^^^
        These parameters are dictionaries and have format:  <term>: [<value 1>, <value 2> ...]
        should_terms: it resembles logical OR
        must_terms: it resembles logical AND
        must_not_terms: it resembles logical NOT

        search_text : string
            Text for FTS(full text search)
        start, end : datetime
            Filter for event creation time
        """
        self.body = self.SearchBody()
        self.body.set_should_terms(should_terms)
        self.body.set_must_terms(must_terms)
        self.body.set_must_not_terms(must_not_terms)
        self.body.set_search_text(search_text)
        self.body.set_timestamp_filter(start, end)
        self.body.prepare()

    def get_events(self, sort='-@timestamp', index='_all', from_=0, size=10, start=None, end=None):
        sort = sort[1:] + ':desc' if sort.startswith('-') else sort + ':asc'
        search_results = self.client.search(index=index, body=self.body, from_=from_, size=size, sort=sort)
        return {
            'events': [r['_source'] for r in search_results['hits']['hits']],
            'total': search_results['hits']['total'],
        }

    def get_count(self, index='_all'):
        count_results = self.client.count(index=index, body=self.body)
        return count_results['count']

    def get_aggregated_by_timestamp_count(self, ranges, index='_all'):
        self.body.set_timestamp_ranges(ranges)
        self.body.prepare()
        search_results = self.client.search(index=index, body=self.body, search_type='count')
        formatted_results = []
        for result in search_results['aggregations']['timestamp_ranges']['buckets']:
            formatted = {'count': result['doc_count']}
            if 'from' in result:
                # Divide by 1000 - because elasticsearch return return timestamp in microseconds
                formatted['start'] = result['from'] / 1000
            if 'to' in result:
                # Divide by 1000 - because elasticsearch return return timestamp in microseconds
                formatted['end'] = result['to'] / 1000
            formatted_results.append(formatted)
        return formatted_results

    def _get_elastisearch_settings(self):
        try:
            elasticsearch_settings = settings.WALDUR_CORE['ELASTICSEARCH']
        except (KeyError, AttributeError):
            raise ElasticsearchClientError(
                'Can not get elasticsearch settings. ELASTICSEARCH item in settings.WALDUR_CORE has '
                'to be defined.')

        required_configuration_fields = {'port', 'host', 'protocol'}
        if not required_configuration_fields.issubset(elasticsearch_settings):
            missing_fields = ','.join(required_configuration_fields - set(elasticsearch_settings))
            raise ElasticsearchClientError(
                'Following configuration items are missing in the "ELASTICSEARCH" section: %s' % missing_fields)

        empty_fields = [field for field in required_configuration_fields if not elasticsearch_settings[field]]
        if empty_fields:
            raise ElasticsearchClientError(
                'Following configuration items are empty in the "ELASTICSEARCH" section: %s' % empty_fields)

        return elasticsearch_settings

    def _get_client(self):
        elasticsearch_settings = self._get_elastisearch_settings()
        if elasticsearch_settings.get('username') and elasticsearch_settings.get('password'):
            path = '%(protocol)s://%(username)s:%(password)s@%(host)s:%(port)s' % elasticsearch_settings
        else:
            path = '%(protocol)s://%(host)s:%(port)s' % elasticsearch_settings
        client = Elasticsearch(
            [str(path)],
            verify_certs=elasticsearch_settings.get('verify_certs', False),
            ca_certs=elasticsearch_settings.get('ca_certs', ''),
        )
        # XXX Workaround for Python Elasticsearch client bugs
        if not elasticsearch_settings.get('verify_certs'):
            # Some parameters are handled incorrectly if verify_certs is false
            # Client's connection pool is the closes place we can fix this
            connection_pool = client.transport.get_connection().pool
            # If ca_certs is not set to 'None' explicitly it will be set to /etc/ssl/certs/ca-certificates.crt
            # which is missing on CentOS.
            # This bug only appears in RPM version of python-urrlib3 (v1.10.2-2 from CentOS Base):
            # http://mirror.centos.org/centos-7/7/os/x86_64/Packages/python-urllib3-1.10.2-2.el7_1.noarch.rpm
            # Upstream handles this situation correctly:
            # https://github.com/shazow/urllib3/blob/1.10.2/urllib3/connectionpool.py#L674L681
            connection_pool.ca_certs = None
            # If verify_certs is set to False no cert_reqs parameter is passed to urrlib3.HTTPSConnectionPool:
            # https://github.com/elastic/elasticsearch-py/blob/1.x/elasticsearch/connection/http_urllib3.py#L46L54
            # Somehow (I couldn't understand why) if cert_reqs is not set to ssl.CERT_NONE explicitly
            # certificate validation still happens -- and fails.
            # To work around the issue, cert_reqs is set to ssl.CERT_NONE explicitly.
            connection_pool.cert_reqs = 0  # ssl.CERT_NONE
        # XXX End of workaround
        return client



from ddt import ddt, data
from rest_framework import status

from waldur_core.structure.tests import factories as structure_factories

from .. import models
from . import factories
from .base_test import BaseCostTrackingTest


@ddt
class PriceListItemListTest(BaseCostTrackingTest):

    def setUp(self):
        super(PriceListItemListTest, self).setUp()
        self.price_list_item = factories.PriceListItemFactory(service=self.service)

    @data('staff', 'owner', 'manager')
    def test_user_with_access_to_service_can_see_services_price_list(self, user):
        self.client.force_authenticate(self.users[user])
        response = self.client.get(factories.PriceListItemFactory.get_list_url())

        self.assertEqual(response.status_code, status.HTTP_200_OK)
        self.assertIn(self.price_list_item.uuid.hex, [el['uuid'] for el in response.data])

    @data('administrator')
    def test_user_without_access_to_service_cannot_see_services_price_list(self, user):
        self.service_project_link.delete()
        self.client.force_authenticate(self.users[user])
        response = self.client.get(factories.PriceListItemFactory.get_list_url())

        self.assertEqual(response.status_code, status.HTTP_200_OK)
        self.assertNotIn(self.price_list_item.uuid.hex, [el['uuid'] for el in response.data])

    def test_price_list_can_be_filtered_by_service(self):
        other_price_list_item = factories.PriceListItemFactory()

        self.client.force_authenticate(self.users['staff'])
        response = self.client.get(
            factories.PriceListItemFactory.get_list_url(),
            data={'service': structure_factories.TestServiceFactory.get_url(self.service)}
        )

        self.assertEqual(response.status_code, status.HTTP_200_OK)
        self.assertIn(self.price_list_item.uuid.hex, [el['uuid'] for el in response.data])
        self.assertNotIn(other_price_list_item.uuid.hex, [el['uuid'] for el in response.data])


@ddt
class PriceListItemCreateTest(BaseCostTrackingTest):

    def setUp(self):
        super(PriceListItemCreateTest, self).setUp()
        self.default_price_list_item = factories.DefaultPriceListItemFactory()
        self.valid_data = {
            'service': structure_factories.TestServiceFactory.get_url(self.service),
            'default_price_list_item': factories.DefaultPriceListItemFactory.get_url(self.default_price_list_item),
            'value': 100,
            'units': 'UAH'
        }

    @data('staff', 'owner')
    def test_user_with_permissions_can_create_price_list_item(self, user):
        self.client.force_authenticate(self.users[user])
        response = self.client.post(factories.PriceListItemFactory.get_list_url(), data=self.valid_data)

        self.assertEqual(response.status_code, status.HTTP_201_CREATED)
        self.assertTrue(models.PriceListItem.objects.filter(
            service=self.service,
            value=self.valid_data['value'],
            default_price_list_item=self.default_price_list_item).exists())

    @data('manager', 'administrator')
    def test_user_without_permissions_cannot_create_price_list_item(self, user):
        self.client.force_authenticate(self.users[user])
        response = self.client.post(factories.PriceListItemFactory.get_list_url(), data=self.valid_data)

        self.assertEqual(response.status_code, status.HTTP_403_FORBIDDEN, str(response.data) + " " + user)
        self.assertFalse(models.PriceListItem.objects.filter(
            service=self.service,
            value=self.valid_data['value'],
            default_price_list_item=self.default_price_list_item).exists())

    def test_if_price_list_item_already_exists_validation_error_is_raised(self):
        self.client.force_authenticate(self.users['staff'])
        response = self.client.post(factories.PriceListItemFactory.get_list_url(), data=self.valid_data)
        self.assertEqual(response.status_code, status.HTTP_201_CREATED)

        response = self.client.post(factories.PriceListItemFactory.get_list_url(), data=self.valid_data)
        self.assertEqual(response.status_code, status.HTTP_400_BAD_REQUEST)


@ddt
class PriceListItemUpdateTest(BaseCostTrackingTest):

    def setUp(self):
        super(PriceListItemUpdateTest, self).setUp()
        self.price_list_item = factories.PriceListItemFactory(service=self.service)

    @data('staff', 'owner')
    def test_user_with_permissions_can_update_price_list_item(self, user):
        self.client.force_authenticate(self.users[user])
        data = {'value': 200}
        response = self.client.patch(factories.PriceListItemFactory.get_url(self.price_list_item), data=data)

        self.assertEqual(response.status_code, status.HTTP_200_OK)
        reread_price_list_item = models.PriceListItem.objects.get(id=self.price_list_item.id)
        self.assertEqual(reread_price_list_item.value, data['value'])

    # We do not execute this test for administrator, because he does not see price estimates at all
    @data('manager')
    def test_user_without_permissions_cannot_update_price_list_item(self, user):
        self.client.force_authenticate(self.users[user])
        data = {'items': [{'name': 'cpu', 'value': 1000, 'units': 'USD per CPU'}]}
        response = self.client.patch(factories.PriceListItemFactory.get_url(self.price_list_item), data=data)

        self.assertEqual(response.status_code, status.HTTP_403_FORBIDDEN)



from __future__ import unicode_literals

from django.utils.translation import ugettext_lazy as _
from rest_framework import status, response

from waldur_core.core import models


class AsyncExecutor(object):
    async_executor = True


class CreateExecutorMixin(AsyncExecutor):
    create_executor = NotImplemented

    def perform_create(self, serializer):
        instance = serializer.save()
        self.create_executor.execute(instance, async=self.async_executor)
        instance.refresh_from_db()


class UpdateExecutorMixin(AsyncExecutor):
    update_executor = NotImplemented

    def perform_update(self, serializer):
        instance = self.get_object()
        # Save all instance fields before update.
        # To avoid additional DB queries - store foreign keys as ids.
        # Warning! M2M fields will be ignored.
        before_update_fields = {f: getattr(instance, f.attname) for f in instance._meta.fields}
        super(UpdateExecutorMixin, self).perform_update(serializer)
        instance.refresh_from_db()
        updated_fields = {f.name for f, v in before_update_fields.items() if v != getattr(instance, f.attname)}
        self.update_executor.execute(instance, async=self.async_executor, updated_fields=updated_fields)
        serializer.instance.refresh_from_db()


class DeleteExecutorMixin(AsyncExecutor):
    delete_executor = NotImplemented

    def destroy(self, request, *args, **kwargs):
        instance = self.get_object()
        self.delete_executor.execute(
            instance, async=self.async_executor, force=instance.state == models.StateMixin.States.ERRED)
        return response.Response(
            {'detail': _('Deletion was scheduled.')}, status=status.HTTP_202_ACCEPTED)


class ExecutorMixin(CreateExecutorMixin, UpdateExecutorMixin, DeleteExecutorMixin):
    """ Executer create/update/delete operation with executor """
    pass


class EagerLoadMixin(object):
    """ Reduce number of requests to DB.

        Serializer should implement static method "eager_load", that selects
        objects that are necessary for serialization.
    """

    def get_queryset(self):
        queryset = super(EagerLoadMixin, self).get_queryset()
        serializer_class = self.get_serializer_class()
        if self.action in ('list', 'retrieve') and hasattr(serializer_class, 'eager_load'):
            queryset = serializer_class.eager_load(queryset)
        return queryset



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations
import waldur_core.core.validators


class Migration(migrations.Migration):

    dependencies = [
        ('contenttypes', '0001_initial'),
        ('monitoring', '0001_initial'),
    ]

    operations = [
        migrations.CreateModel(
            name='ResourceItem',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('name', models.CharField(max_length=150, verbose_name='name', validators=[waldur_core.core.validators.validate_name])),
                ('object_id', models.PositiveIntegerField()),
                ('value', models.FloatField()),
                ('content_type', models.ForeignKey(to='contenttypes.ContentType')),
            ],
            options={
            },
            bases=(models.Model,),
        ),
        migrations.CreateModel(
            name='ResourceSla',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('object_id', models.PositiveIntegerField()),
                ('period', models.CharField(max_length=10)),
                ('value', models.DecimalField(null=True, max_digits=11, decimal_places=4, blank=True)),
                ('agreed_value', models.DecimalField(null=True, max_digits=11, decimal_places=4, blank=True)),
                ('content_type', models.ForeignKey(to='contenttypes.ContentType')),
            ],
            options={
            },
            bases=(models.Model,),
        ),
        migrations.CreateModel(
            name='ResourceSlaStateTransition',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('object_id', models.PositiveIntegerField()),
                ('period', models.CharField(max_length=10)),
                ('timestamp', models.IntegerField()),
                ('state', models.BooleanField(default=False, help_text="If state is True resource became available")),
                ('content_type', models.ForeignKey(to='contenttypes.ContentType')),
            ],
            options={
            },
            bases=(models.Model,),
        ),
        migrations.AlterUniqueTogether(
            name='monitoringitem',
            unique_together=None,
        ),
        migrations.RemoveField(
            model_name='monitoringitem',
            name='content_type',
        ),
        migrations.DeleteModel(
            name='MonitoringItem',
        ),
        migrations.AlterUniqueTogether(
            name='resourceslastatetransition',
            unique_together=set([('timestamp', 'period', 'content_type', 'object_id')]),
        ),
        migrations.AlterUniqueTogether(
            name='resourcesla',
            unique_together=set([('period', 'content_type', 'object_id')]),
        ),
        migrations.AlterUniqueTogether(
            name='resourceitem',
            unique_together=set([('name', 'content_type', 'object_id')]),
        ),
    ]



#!/usr/bin/env python
import os
import sys


def main():
    os.environ.setdefault('DJANGO_SETTINGS_MODULE', 'waldur_core.server.settings')
    from django.core.management import execute_from_command_line
    execute_from_command_line(sys.argv)

if __name__ == '__main__':
    main()



from django.utils import six

from waldur_core.core.models import User
from waldur_core.logging.loggers import EventLogger, event_logger
from waldur_core.structure import models


class CustomerEventLogger(EventLogger):
    customer = models.Customer

    class Meta:
        event_types = ('customer_deletion_succeeded',
                       'customer_update_succeeded',
                       'customer_creation_succeeded')
        event_groups = {
            'customers': event_types,
        }


class ProjectEventLogger(EventLogger):
    project = models.Project

    class Meta:
        event_types = ('project_deletion_succeeded',
                       'project_update_succeeded',
                       'project_creation_succeeded')
        event_groups = {
            'projects': event_types,
        }


class CustomerRoleEventLogger(EventLogger):
    customer = models.Customer
    affected_user = User
    structure_type = six.text_type
    role_name = six.text_type

    class Meta:
        event_types = 'role_granted', 'role_revoked', 'role_updated'
        event_groups = {
            'customers': event_types,
            'users': event_types,
        }


class ProjectRoleEventLogger(EventLogger):
    project = models.Project
    affected_user = User
    structure_type = six.text_type
    role_name = six.text_type

    class Meta:
        event_types = 'role_granted', 'role_revoked', 'role_updated'
        event_groups = {
            'projects': event_types,
            'users': event_types,
        }


class UserOrganizationEventLogger(EventLogger):
    affected_user = User
    affected_organization = six.text_type

    class Meta:
        event_types = ('user_organization_claimed',
                       'user_organization_approved',
                       'user_organization_rejected',
                       'user_organization_removed')


class ResourceEventLogger(EventLogger):
    resource = models.ResourceMixin

    class Meta:
        event_types = (
            'resource_start_scheduled',
            'resource_start_succeeded',
            'resource_start_failed',

            'resource_stop_scheduled',
            'resource_stop_succeeded',
            'resource_stop_failed',

            'resource_restart_scheduled',
            'resource_restart_succeeded',
            'resource_restart_failed',

            'resource_creation_scheduled',
            'resource_creation_succeeded',
            'resource_creation_failed',

            'resource_import_succeeded',
            'resource_update_succeeded',

            'resource_deletion_scheduled',
            'resource_deletion_succeeded',
            'resource_deletion_failed',
        )
        event_groups = {
            'resources': event_types,
        }


event_logger.register('customer_role', CustomerRoleEventLogger)
event_logger.register('project_role', ProjectRoleEventLogger)
event_logger.register('user_organization', UserOrganizationEventLogger)
event_logger.register('customer', CustomerEventLogger)
event_logger.register('project', ProjectEventLogger)
event_logger.register('resource', ResourceEventLogger)



from __future__ import unicode_literals

import unittest

from django.utils import timezone
from freezegun import freeze_time
from rest_framework import status
from rest_framework import test

from waldur_core.core.models import User
from waldur_core.structure.models import CustomerRole
from waldur_core.structure.serializers import PasswordSerializer
from waldur_core.structure.tests import factories

from . import fixtures


class UserPermissionApiTest(test.APITransactionTestCase):
    def setUp(self):
        self.users = {
            'staff': factories.UserFactory(is_staff=True, agreement_date=timezone.now()),
            'owner': factories.UserFactory(agreement_date=timezone.now()),
            'not_owner': factories.UserFactory(agreement_date=timezone.now()),
        }

    # List filtration tests
    def test_anonymous_user_cannot_list_accounts(self):
        response = self.client.get(factories.UserFactory.get_list_url())
        self.assertEqual(response.status_code, status.HTTP_401_UNAUTHORIZED)

    def test_authorized_user_can_list_accounts(self):
        self.client.force_authenticate(self.users['owner'])

        response = self.client.get(factories.UserFactory.get_list_url())
        self.assertEqual(response.status_code, status.HTTP_200_OK)

    def test_staff_user_can_list_accounts(self):
        self.client.force_authenticate(user=self.users['staff'])

        response = self.client.get(factories.UserFactory.get_list_url())
        self.assertEqual(response.status_code, status.HTTP_200_OK)

    def test_staff_can_see_token_in_the_list(self):
        self.client.force_authenticate(self.users['staff'])

        response = self.client.get(factories.UserFactory.get_list_url())
        self.assertEqual(response.status_code, status.HTTP_200_OK)
        self.assertEqual(len(response.data), len(self.users))
        self.assertIsNotNone(response.data[0]['token'])

    def test_staff_can_see_token_and_its_lifetime_of_the_other_user(self):
        self.client.force_authenticate(self.users['staff'])

        response = self.client.get(factories.UserFactory.get_url(self.users['owner']))
        self.assertEqual(response.status_code, status.HTTP_200_OK)
        self.assertIsNotNone(response.data['token'])
        self.assertIn('token_lifetime', response.data)

    def test_owner_cannot_see_token_and_its_lifetime_field_in_the_list_of_users(self):
        self.client.force_authenticate(self.users['owner'])

        response = self.client.get(factories.UserFactory.get_list_url())
        self.assertEqual(response.status_code, status.HTTP_200_OK)
        self.assertEqual(len(response.data), 2)
        self.assertIsNot('token', response.data[0])
        self.assertIsNot('token_lifetime', response.data[0])

    def test_owner_can_see_his_token_and_its_lifetime(self):
        self.client.force_authenticate(self.users['owner'])

        response = self.client.get(factories.UserFactory.get_url(self.users['owner']))
        self.assertEqual(response.status_code, status.HTTP_200_OK)
        self.assertIsNotNone(response.data['token'])
        self.assertIn('token_lifetime', response.data)

    def test_owner_cannot_see_token_and_its_lifetime_of_the_other_user(self):
        self.client.force_authenticate(self.users['owner'])

        response = self.client.get(factories.UserFactory.get_url(self.users['not_owner']))
        self.assertEqual(response.status_code, status.HTTP_200_OK)
        self.assertNotIn('token', response.data)
        self.assertNotIn('token_lifetime', response.data)

    def test_user_can_see_his_token_via_current_filter(self):
        self.client.force_authenticate(self.users['owner'])

        response = self.client.get(factories.UserFactory.get_list_url(), {'current': True})
        self.assertEqual(response.status_code, status.HTTP_200_OK)
        self.assertEqual(1, len(response.data))
        self.assertIsNotNone('token', response.data[0])
        self.assertIsNotNone('token_lifetime', response.data[0])

    # Creation tests
    def test_anonymous_user_cannot_create_account(self):
        data = self._get_valid_payload()

        response = self.client.post(factories.UserFactory.get_list_url(), data)
        self.assertEqual(response.status_code, status.HTTP_401_UNAUTHORIZED)

    def test_authorized_user_cannot_create_account(self):
        self.client.force_authenticate(self.users['owner'])

        data = self._get_valid_payload()

        response = self.client.post(factories.UserFactory.get_list_url(), data)
        self.assertEqual(response.status_code, status.HTTP_403_FORBIDDEN)

    def test_staff_user_can_create_account(self):
        self.client.force_authenticate(self.users['staff'])

        data = self._get_valid_payload()

        response = self.client.post(factories.UserFactory.get_list_url(), data)
        self.assertEqual(response.status_code, status.HTTP_201_CREATED)

        created_user = User.objects.filter(username=data['username']).first()
        self.assertIsNotNone(created_user, 'User should have been created')

    def test_staff_user_cannot_set_civil_number_upon_account_creation(self):
        self.client.force_authenticate(self.users['staff'])

        data = self._get_valid_payload()
        data['civil_number'] = 'foobar'

        response = self.client.post(factories.UserFactory.get_list_url(), data)
        self.assertEqual(response.status_code, status.HTTP_201_CREATED)

        created_user = User.objects.get(username=data['username'])
        self.assertIsNone(created_user.civil_number, "User's civil_number should be unset")

    # Manipulation tests
    def test_user_can_change_his_account_email(self):
        data = {
            'email': 'example@example.com',
        }

        self._ensure_user_can_change_field(self.users['owner'], 'email', data)

    def test_user_cannot_change_other_account_email(self):
        data = {
            'email': 'example@example.com',
        }

        self._ensure_user_cannot_change_field(self.users['owner'], 'email', data)

    def test_staff_user_cannot_change_civil_number(self):
        self.client.force_authenticate(self.users['staff'])

        user = factories.UserFactory()

        data = self._get_valid_payload(user)
        data['civil_number'] = 'foobar'

        self.client.put(factories.UserFactory.get_url(user), data)

        reread_user = User.objects.get(username=data['username'])
        self.assertEqual(reread_user.civil_number, user.civil_number,
                         "User's civil_number should be left intact")

    @unittest.skip('Disabling as organization is temporary a read-only field.')
    def test_user_can_change_his_account_organization(self):
        data = {
            'organization': 'test',
            'email': 'example@example.com',
        }

        self._ensure_user_can_change_field(self.users['owner'], 'organization', data)

    def test_user_cannot_change_other_account_organization(self):
        data = {
            'organization': 'test',
            'email': 'example@example.com',
        }

        self._ensure_user_cannot_change_field(self.users['owner'], 'organization', data)

    def test_user_can_change_his_token_lifetime(self):
        data = {
            'email': 'example@example.com',
            'token_lifetime': 100,
        }

        self._ensure_user_can_change_field(self.users['owner'], 'token_lifetime', data)

    def test_user_cannot_change_other_token_lifetime(self):
        data = {
            'email': 'example@example.com',
            'token_lifetime': 100,
        }

        self._ensure_user_cannot_change_field(self.users['owner'], 'token_lifetime', data)

    def test_staff_user_can_change_any_accounts_fields(self):
        self.client.force_authenticate(user=self.users['staff'])
        data = self._get_valid_payload()

        response = self.client.put(factories.UserFactory.get_url(self.users['staff']), data)
        self.assertEqual(response.status_code, status.HTTP_200_OK)

    # Password changing tests
    def test_user_can_change_his_account_password(self):
        self.client.force_authenticate(self.users['owner'])

        data = {'password': 'nQvqHzeP123'}

        response = self.client.post(factories.UserFactory.get_password_url(self.users['owner']), data)
        self.assertEqual(response.status_code, status.HTTP_200_OK)
        self.assertEqual('Password has been successfully updated.', response.data['detail'])

        user = User.objects.get(uuid=self.users['owner'].uuid)
        self.assertTrue(user.check_password(data['password']))

    def test_user_cannot_change_other_account_password(self):
        self.client.force_authenticate(self.users['not_owner'])

        data = {'password': 'nQvqHzeP123'}

        response = self.client.post(factories.UserFactory.get_password_url(self.users['owner']), data)
        self.assertEqual(response.status_code, status.HTTP_403_FORBIDDEN)

        user = User.objects.get(uuid=self.users['owner'].uuid)
        self.assertFalse(user.check_password(data['password']))

    def test_staff_user_can_change_any_accounts_password(self):
        self.client.force_authenticate(self.users['staff'])

        data = {'password': 'nQvqHzeP123'}

        for user in self.users:
            response = self.client.post(factories.UserFactory.get_password_url(self.users[user]), data)
            self.assertEqual(response.status_code, status.HTTP_200_OK)

            user = User.objects.get(uuid=self.users[user].uuid)
            self.assertTrue(user.check_password(data['password']))

    # Deletion tests
    def user_cannot_delete_his_account(self):
        self._ensure_user_cannot_delete_account(self.users['owner'], self.users['owner'])

    def user_cannot_delete_other_account(self):
        self._ensure_user_cannot_delete_account(self.users['not_owner'], self.users['owner'])

    def test_staff_user_can_delete_any_account(self):
        self.client.force_authenticate(user=self.users['staff'])

        for user in self.users:
            response = self.client.delete(factories.UserFactory.get_url(self.users[user]))
            self.assertEqual(response.status_code, status.HTTP_204_NO_CONTENT)

    # Helper methods
    def _get_valid_payload(self, account=None):
        account = account or factories.UserFactory.build()

        return {
            'username': account.username,
            'email': account.email,
            'full_name': account.full_name,
            'native_name': account.native_name,
            'is_staff': account.is_staff,
            'is_active': account.is_active,
        }

    def _get_null_payload(self, account=None):
        account = account or factories.UserFactory.build()

        return {
            'username': account.username,
            'email': account.email,
            'full_name': None,
            'native_name': None,
            'phone_number': None,
            'description': None,
            'is_staff': account.is_staff,
            'is_active': account.is_active,
        }

    def _ensure_user_can_change_field(self, user, field_name, data):
        self.client.force_authenticate(user)

        response = self.client.put(factories.UserFactory.get_url(user), data)
        self.assertEqual(response.status_code, status.HTTP_200_OK)

        new_value = getattr(User.objects.get(uuid=user.uuid), field_name)
        self.assertEqual(new_value, data[field_name])

    def _ensure_user_cannot_change_field(self, user, field_name, data):
        self.client.force_authenticate(user)

        response = self.client.put(factories.UserFactory.get_url(self.users['not_owner']), data)
        self.assertEqual(response.status_code, status.HTTP_403_FORBIDDEN)

        new_value = getattr(User.objects.get(uuid=self.users['not_owner'].uuid), field_name)
        self.assertNotEqual(new_value, data[field_name])

    def _ensure_user_cannot_delete_account(self, user, account):
        self.client.force_authenticate(user=user)

        response = self.client.delete(factories.UserFactory.get_url(account))
        self.assertEqual(response.status_code, status.HTTP_403_FORBIDDEN)


class PasswordSerializerTest(unittest.TestCase):
    def test_password_must_be_at_least_7_characters_long(self):
        data = {'password': '123abc'}

        serializer = PasswordSerializer(data=data)

        self.assertFalse(serializer.is_valid())
        self.assertDictContainsSubset(
            {'password': ['Ensure this field has at least 7 characters.']}, serializer.errors)

    def test_password_must_contain_digits(self):
        data = {'password': 'abcdefg'}

        serializer = PasswordSerializer(data=data)

        self.assertFalse(serializer.is_valid())
        self.assertDictContainsSubset(
            {'password': ['Ensure this field has at least one digit.']}, serializer.errors)

    def test_password_must_contain_letters(self):
        data = {'password': '1234567'}

        serializer = PasswordSerializer(data=data)

        self.assertFalse(serializer.is_valid())
        self.assertDictContainsSubset(
            {'password': ['Ensure this field has at least one latin letter.']},
            serializer.errors)

    def test_password_must_not_be_blank(self):
        data = {'password': ''}

        serializer = PasswordSerializer(data=data)

        self.assertFalse(serializer.is_valid())
        self.assertDictContainsSubset(
            {'password': ['This field may not be blank.']}, serializer.errors)


class UserFilterTest(test.APITransactionTestCase):

    def test_user_list_can_be_filtered(self):
        supported_filters = [
            'full_name',
            'native_name',
            'organization',
            'email',
            'phone_number',
            'description',
            'job_title',
            'username',
            'civil_number',
            'is_active',
        ]
        user = factories.UserFactory(is_staff=True)
        user_that_should_be_found = factories.UserFactory(
            native_name='',
            organization='',
            email='none@example.com',
            phone_number='',
            description='',
            job_title='',
            username='',
            civil_number='',
            is_active=False,
        )
        self.client.force_authenticate(user)
        url = factories.UserFactory.get_list_url()
        user_url = factories.UserFactory.get_url(user)
        user_that_should_not_be_found_url = factories.UserFactory.get_url(user_that_should_be_found)

        for field in supported_filters:
            response = self.client.get(url, data={field: getattr(user, field)})
            self.assertContains(response, user_url)
            self.assertNotContains(response, user_that_should_not_be_found_url)

    def test_user_list_can_be_filtered_by_fields_with_partial_matching(self):
        supported_filters = [
            'full_name',
            'native_name',
            'email',
            'job_title',
        ]
        user = factories.UserFactory(is_staff=True)
        self.client.force_authenticate(user)
        url = factories.UserFactory.get_list_url()
        user_url = factories.UserFactory.get_url(user)

        for field in supported_filters:
            response = self.client.get(url, data={field: getattr(user, field)[:-1]})
            self.assertContains(response, user_url)


# TODO: temporary flow. Remove once a proper solution is in place.
class UserOrganizationApprovalApiTest(test.APITransactionTestCase):
    def setUp(self):
        self.users = {
            'no_role': factories.UserFactory(organization=""),
            'user_with_request_to_a_customer': factories.UserFactory(),
            'owner': factories.UserFactory(),
            'owner_of_another_customer': factories.UserFactory(),
            'staff': factories.UserFactory(is_staff=True),
        }
        self.customer = factories.CustomerFactory()
        self.customer.add_user(self.users['owner'], CustomerRole.OWNER)

        self.another_customer = factories.CustomerFactory()
        self.another_customer.add_user(self.users['owner_of_another_customer'], CustomerRole.OWNER)

        self.users['user_with_request_to_a_customer'].organization = self.customer.abbreviation
        self.users['user_with_request_to_a_customer'].organization_approved = False
        self.users['user_with_request_to_a_customer'].save()

    # positive
    def test_user_can_claim_organization_membership_if_organization_is_empty(self):
        user = self.users['no_role']
        self.client.force_authenticate(user)
        url = factories.UserFactory.get_url(user, action='claim_organization')
        client_url = factories.UserFactory.get_url(user)

        response = self.client.post(url, data={'organization': self.customer.abbreviation})
        self.assertEquals(response.status_code, status.HTTP_200_OK, response.data)

        #check the status of the claim
        response = self.client.get(client_url)
        self.assertDictContainsSubset(
            {'organization': self.customer.abbreviation,
             'organization_approved': False},
            response.data
        )

    def test_user_can_remove_organization_before_it_is_approved(self):
        user = self.users['user_with_request_to_a_customer']
        self.client.force_authenticate(user)
        url = factories.UserFactory.get_url(user, action='remove_organization')
        client_url = factories.UserFactory.get_url(user)

        response = self.client.post(url)
        self.assertEquals(response.status_code, status.HTTP_200_OK, response.data)

        #check the status of the claim
        response = self.client.get(client_url)
        self.assertDictContainsSubset(
            {'organization': '',
             'organization_approved': False},
            response.data
        )

    def test_user_see_status_of_his_claim_for_organization_membership(self):
        user = self.users['no_role']
        self.client.force_authenticate(user)
        url = factories.UserFactory.get_url(user)

        response = self.client.get(url)
        self.assertTrue(response.status_code, status.HTTP_200_OK)
        self.assertDictContainsSubset(
            {'organization': user.organization,
             'organization_approved': user.organization_approved},
            response.data
        )

    def test_user_who_is_customer_owner_can_see_requests_to_join_his_customer(self):
        self._can_see_status_of_user_claim_for_organization_membership(self.users['owner'])

    def test_user_who_is_customer_owner_can_approve_user_request_to_join_his_customer(self):
        self._can_approve_user_request_to_join_a_customer(self.users['owner'])

    def test_user_who_is_customer_owner_can_reject_user_request_to_join_his_customer(self):
        self._can_reject_user_request_to_join_a_customer(self.users['owner'])

    def test_user_who_is_customer_owner_can_remove_user_from_his_customer_organization(self):
        self._can_remove_user_from_his_customer_organization(self.users['owner'])

    # negative
    def test_user_cannot_claim_organization_membership_if_he_has_approved_one(self):
        user = self.users['user_with_request_to_a_customer']
        user.organization_approved = True
        user.save()

        self.client.force_authenticate(user)
        url = factories.UserFactory.get_url(user, action='claim_organization')

        response = self.client.post(url, data={'organization': self.customer.abbreviation})
        self.assertEquals(response.status_code, status.HTTP_409_CONFLICT, response.data)

    def test_user_cannot_claim_empty_organization_name(self):
        user = self.users['no_role']
        self.client.force_authenticate(user)
        url = factories.UserFactory.get_url(user, action='claim_organization')

        response = self.client.post(url, data={'organization': ''})
        self.assertEquals(response.status_code, status.HTTP_400_BAD_REQUEST)
        self.assertDictContainsSubset({'organization': ['This field may not be blank.']}, response.data)

    def test_user_who_is_not_staff_cannot_claim_organization_membership_of_another_user(self):
        user = self.users['user_with_request_to_a_customer']
        self.client.force_authenticate(user)
        url = factories.UserFactory.get_url(self.users['no_role'], action='claim_organization')

        response = self.client.post(url, data={'organization': self.customer.abbreviation})
        self.assertEquals(response.status_code, status.HTTP_403_FORBIDDEN, response.data)

    def test_user_who_is_not_customer_owner_cannot_approve_user_request_to_join_customer(self):
        user = self.users['no_role']
        self.client.force_authenticate(user)
        url = factories.UserFactory.get_url(self.users['user_with_request_to_a_customer'],
                                            action='approve_organization')

        response = self.client.post(url)
        self.assertEquals(response.status_code, status.HTTP_403_FORBIDDEN, response.data)

    def test_user_who_is_not_customer_owner_cannot_reject_user_request_to_join_customer(self):
        user = self.users['no_role']
        self.client.force_authenticate(user)
        url = factories.UserFactory.get_url(self.users['user_with_request_to_a_customer'],
                                            action='reject_organization')

        response = self.client.post(url)
        self.assertEquals(response.status_code, status.HTTP_403_FORBIDDEN, response.data)

    def test_user_who_is_not_customer_owner_cannot_remove_user_from_customer_organization(self):
        user = self.users['no_role']
        self.client.force_authenticate(user)
        url = factories.UserFactory.get_url(self.users['user_with_request_to_a_customer'],
                                            action='remove_organization')

        response = self.client.post(url)
        self.assertEquals(response.status_code, status.HTTP_403_FORBIDDEN, response.data)

    # staff
    def test_staff_can_see_status_of_user_claim_for_organization_membership(self):
        self._can_see_status_of_user_claim_for_organization_membership(self.users['staff'])

    def test_staff_can_see_requests_to_join_customer(self):
        user = self.users['staff']
        self.client.force_authenticate(user)
        user_url = factories.UserFactory.get_url(self.users['user_with_request_to_a_customer'])
        user_list = factories.UserFactory.get_list_url()
        response = self.client.get(user_list, data={
            'organization': self.customer.abbreviation,
            'organization_approved': False,
        })
        self.assertTrue(response.status_code, status.HTTP_200_OK)
        self.assertContains(response, user_url)
        self.assertTrue(len(response.data) == 1)

    def test_staff_can_approve_user_request_to_join_customer(self):
        self._can_approve_user_request_to_join_a_customer(self.users['staff'])

    def test_staff_can_reject_user_request_to_join_customer(self):
        self._can_reject_user_request_to_join_a_customer(self.users['staff'])

    def test_staff_can_remove_user_from_customer_organization(self):
        self._can_remove_user_from_his_customer_organization(self.users['staff'])

    # helper methods
    def _can_see_status_of_user_claim_for_organization_membership(self, request_user):
        self.client.force_authenticate(request_user)
        user = self.users['no_role']
        url = factories.UserFactory.get_url(user)

        response = self.client.get(url)
        self.assertTrue(response.status_code, status.HTTP_200_OK)
        self.assertDictContainsSubset(
            {'organization': user.organization,
             'organization_approved': user.organization_approved},
            response.data
        )

    def _can_approve_user_request_to_join_a_customer(self, request_user):
        self.client.force_authenticate(request_user)
        url = factories.UserFactory.get_url(self.users['user_with_request_to_a_customer'],
                                            action='approve_organization')
        response = self.client.post(url)

        self.assertEquals(response.status_code, status.HTTP_200_OK)

        # reload user from db
        candidate_user = User.objects.get(username=self.users['user_with_request_to_a_customer'].username)
        self.assertTrue(candidate_user.organization == self.customer.abbreviation)
        self.assertTrue(candidate_user.organization_approved, 'Organization is not approved')

    def _can_reject_user_request_to_join_a_customer(self, request_user):
        self.client.force_authenticate(request_user)
        url = factories.UserFactory.get_url(self.users['user_with_request_to_a_customer'],
                                            action='reject_organization')
        response = self.client.post(url)

        self.assertEquals(response.status_code, status.HTTP_200_OK)

        # reload user from db
        candidate_user = User.objects.get(username=self.users['user_with_request_to_a_customer'].username)
        self.assertTrue(candidate_user.organization == "")
        self.assertFalse(candidate_user.organization_approved, 'Organization is not approved')

    def _can_remove_user_from_his_customer_organization(self, request_user):
        self.client.force_authenticate(request_user)
        url = factories.UserFactory.get_url(self.users['user_with_request_to_a_customer'],
                                            action='remove_organization')
        response = self.client.post(url)

        self.assertEquals(response.status_code, status.HTTP_200_OK)

        # reload user from db
        candidate_user = User.objects.get(username=self.users['user_with_request_to_a_customer'].username)
        self.assertTrue(candidate_user.organization == "")
        self.assertFalse(candidate_user.organization_approved, 'Organization is not approved')


class CustomUsersFilterTest(test.APITransactionTestCase):

    def setUp(self):
        fixture = fixtures.ProjectFixture()
        self.customer1 = fixture.customer
        self.project1 = fixture.project
        self.staff = fixture.staff
        self.owner1 = fixture.owner
        self.manager1 = fixture.manager

        fixture2 = fixtures.ProjectFixture()
        self.customer2 = fixture2.customer
        self.project2 = fixture2.project
        self.owner2 = fixture2.owner
        self.manager2 = fixture2.manager

        self.client.force_authenticate(self.staff)
        self.url = factories.UserFactory.get_list_url()

    def test_filter_user_by_customer(self):
        response = self.client.get(self.url, {'customer_uuid': self.customer1.uuid.hex})
        self.assertEquals(response.status_code, status.HTTP_200_OK)

        actual = [user['uuid'] for user in response.data]
        expected = [self.owner1.uuid.hex, self.manager1.uuid.hex]
        self.assertEquals(actual, expected)

    def test_filter_user_by_project(self):
        response = self.client.get(self.url, {'project_uuid': self.project1.uuid.hex})
        self.assertEquals(response.status_code, status.HTTP_200_OK)

        actual = [user['uuid'] for user in response.data]
        expected = [self.manager1.uuid.hex]
        self.assertEquals(actual, expected)


@freeze_time('2017-01-19 00:00:00')
class UserUpdateTest(test.APITransactionTestCase):
    def setUp(self):
        fixture = fixtures.UserFixture()
        self.user = fixture.user
        self.client.force_authenticate(self.user)
        self.url = factories.UserFactory.get_url(self.user)

        self.invalid_payload = {
            'email': 'updatedmail@example.com',
        }
        self.valid_payload = dict(agree_with_policy=True, **self.invalid_payload)

    def test_if_user_did_not_accept_policy_he_can_not_update_his_profile(self):
        response = self.client.put(self.url, self.invalid_payload)
        self.assertEquals(response.status_code, status.HTTP_400_BAD_REQUEST)
        self.assertEquals(response.data['agree_with_policy'], ['User must agree with the policy.'])

    def test_if_user_already_accepted_policy_he_can_update_his_profile(self):
        self.user.agreement_date = timezone.now()
        self.user.save()

        response = self.client.put(self.url, self.invalid_payload)
        self.assertEquals(response.status_code, status.HTTP_200_OK)

    def test_if_user_accepts_policy_he_can_update_his_profile(self):
        response = self.client.put(self.url, self.valid_payload)
        self.assertEquals(response.status_code, status.HTTP_200_OK)

        self.user.refresh_from_db()
        self.assertEquals(self.user.email, self.valid_payload['email'])

    def test_if_user_accepts_policy_agreement_data_is_updated(self):
        response = self.client.put(self.url, self.valid_payload)
        self.assertEquals(response.status_code, status.HTTP_200_OK)

        self.user.refresh_from_db()
        self.assertAlmostEqual(self.user.agreement_date, timezone.now())

    def test_email_should_be_unique_and_error_should_be_specific_for_field(self):
        email = self.invalid_payload['email']
        factories.UserFactory(email=email)

        response = self.client.put(self.url, self.valid_payload)

        self.assertEquals(response.status_code, status.HTTP_400_BAD_REQUEST)
        self.assertEquals(response.data['email'], ['User with email "%s" already exists.' % email])

    def test_token_lifetime_cannot_be_less_than_60_seconds(self):
        self.valid_payload['token_lifetime'] = 59

        response = self.client.put(self.url, self.valid_payload)
        self.assertEquals(response.status_code, status.HTTP_400_BAD_REQUEST)
        self.assertIn('token_lifetime', response.data)



# The dancing with the function and its deletion is done
# to keep the namespace clean: only __version__ is going to be exposed.


# https://gist.github.com/edufelipe/1027906
def _check_output(*popenargs, **kwargs):
    r"""Run command with arguments and return its output as a byte string.

    Backported from Python 2.7 as it's implemented as pure python on stdlib.

    >>> _check_output(['/usr/bin/python', '--version'])
    Python 2.6.2
    """
    import subprocess  # nosec
    process = subprocess.Popen(stdout=subprocess.PIPE, *popenargs, **kwargs)  # nosec
    output, unused_err = process.communicate()
    retcode = process.poll()
    if retcode:
        cmd = kwargs.get("args")
        if cmd is None:
            cmd = popenargs[0]
        error = subprocess.CalledProcessError(retcode, cmd)
        error.output = output
        raise error
    return output


def _get_version(package_name='waldur_core'):
    import pkg_resources

    # Based on http://stackoverflow.com/a/17638236/175349
    # and https://github.com/pwaller/__autoversion__/blob/master/__autoversion__.py

    try:
        return pkg_resources.get_distribution(package_name).version
    except pkg_resources.DistributionNotFound:
        import os.path
        import re
        import subprocess  # nosec

        repo_dir = os.path.join(os.path.dirname(__file__), os.path.pardir)

        try:
            with open(os.devnull, 'w') as DEV_NULL:
                description = _check_output(
                    ['git', 'describe', '--tags', '--dirty=.dirty'],
                    cwd=repo_dir, stderr=DEV_NULL
                ).strip()

            v = re.search(r'-[0-9]+-', description)
            if v is not None:
                # Replace -n- with -branchname-n-
                # branch = r"-{0}-\1-".format(cls.get_branch(path))
                description, _ = re.subn('-([0-9]+)-', r'+\1.', description, 1)

            if description[0] == 'v':
                description = description[1:]

            return description
        except (OSError, subprocess.CalledProcessError):
            return 'unknown'


__version__ = _get_version()



from ddt import ddt, data
from rest_framework import test, status

from waldur_core.structure import models

from . import fixtures, factories


class BaseServiceCertificationTest(test.APITransactionTestCase):

    def setUp(self):
        self.fixture = fixtures.ProjectFixture()


@ddt
class ServiceCertificationRetrieveTest(BaseServiceCertificationTest):

    def setUp(self):
        super(ServiceCertificationRetrieveTest, self).setUp()
        self.url = factories.ServiceCertificationFactory.get_list_url()

    @data('user', 'global_support', 'owner', 'manager', 'admin', 'staff')
    def test_user_can_see_list_of_available_certifications(self, user):
        self.client.force_authenticate(getattr(self.fixture, user))
        certification = factories.ServiceCertificationFactory()

        response = self.client.get(self.url)

        self.assertEqual(len(response.data), 1)
        self.assertEqual(response.data[0]['uuid'], certification.uuid.hex)


@ddt
class ServiceCertificationCreateTest(BaseServiceCertificationTest):

    def setUp(self):
        super(ServiceCertificationCreateTest, self).setUp()
        self.url = factories.ServiceCertificationFactory.get_list_url()

    @data('staff')
    def test_staff_can_create_certification(self, user):
        self.client.force_authenticate(getattr(self.fixture, user))
        payload = {
            'name': 'ISO3728',
            'link': 'http://www.iso.org/unknown',
            'description': 'this is a description',
        }

        response = self.client.post(self.url, payload)

        self.assertEqual(response.status_code, status.HTTP_201_CREATED)
        certification = models.ServiceCertification.objects.get(name=payload['name'])
        self.assertEqual(response.data['name'], payload['name'])
        self.assertEqual(certification.name, payload['name'])
        self.assertEqual(response.data['link'], payload['link'])
        self.assertEqual(certification.link, payload['link'])
        self.assertEqual(response.data['description'], payload['description'])
        self.assertEqual(certification.description, payload['description'])

    @data('user', 'global_support', 'owner', 'manager', 'admin')
    def test_only_staff_can_create_certification(self, user):
        self.client.force_authenticate(getattr(self.fixture, user))
        payload = {
            'name': 'ISO3728',
            'link': 'http://www.iso.org/unknown',
            'description': 'this is a description',
        }

        response = self.client.post(self.url, payload)

        self.assertEqual(response.status_code, status.HTTP_403_FORBIDDEN)


@ddt
class ServiceCertificationUpdateTest(BaseServiceCertificationTest):

    def setUp(self):
        super(ServiceCertificationUpdateTest, self).setUp()
        self.certification = factories.ServiceCertificationFactory()
        self.url = factories.ServiceCertificationFactory.get_url(self.certification)

    @data('staff')
    def test_staff_can_update_certification(self, user):
        self.client.force_authenticate(getattr(self.fixture, user))
        expected_name = 'new_name'

        response = self.client.put(self.url, {'name': expected_name})

        self.assertEqual(response.status_code, status.HTTP_200_OK)
        self.certification.refresh_from_db()
        self.assertEqual(self.certification.name, expected_name)

    @data('admin', 'manager', 'global_support', 'user')
    def test_only_staff_can_update_certification(self, user):
        self.client.force_authenticate(getattr(self.fixture, user))

        response = self.client.put(self.url, {'name': 'new_name'})

        self.assertEqual(response.status_code, status.HTTP_403_FORBIDDEN)


@ddt
class ServiceCertificationDeleteTest(BaseServiceCertificationTest):

    def setUp(self):
        super(ServiceCertificationDeleteTest, self).setUp()
        self.certification = factories.ServiceCertificationFactory()
        self.url = factories.ServiceCertificationFactory.get_url(self.certification)

    @data('staff')
    def test_staff_can_delete_certification(self, user):
        self.client.force_authenticate(getattr(self.fixture, user))

        response = self.client.delete(self.url)

        self.assertEqual(response.status_code, status.HTTP_204_NO_CONTENT)
        with self.assertRaises(models.ServiceCertification.DoesNotExist):
            self.certification.refresh_from_db()

    @data('admin', 'manager', 'global_support', 'user')
    def test_only_staff_can_delete_certification(self, user):
        self.client.force_authenticate(getattr(self.fixture, user))

        response = self.client.delete(self.url)

        self.assertEqual(response.status_code, status.HTTP_403_FORBIDDEN)
        self.assertTrue(models.ServiceCertification.objects.filter(pk=self.certification.pk).exists())



from __future__ import unicode_literals

from django.utils.translation import ugettext_lazy as _
from rest_framework import exceptions as rf_exceptions, decorators, response, status
from rest_framework import mixins
from rest_framework import viewsets
from reversion import revisions as reversion
from reversion.models import Version

from waldur_core.core.pagination import UnlimitedLinkHeaderPagination
from waldur_core.core.serializers import HistorySerializer
from waldur_core.core.utils import datetime_to_timestamp
from waldur_core.quotas import models, serializers, filters, exceptions


class QuotaViewSet(mixins.UpdateModelMixin,
                   viewsets.ReadOnlyModelViewSet):
    queryset = models.Quota.objects.all()
    serializer_class = serializers.QuotaSerializer
    lookup_field = 'uuid'
    # XXX: Remove a custom pagination class once the quota calculation has been made more efficient
    pagination_class = UnlimitedLinkHeaderPagination
    filter_class = filters.QuotaFilterSet

    def get_queryset(self):
        return models.Quota.objects.filtered_for_user(self.request.user)

    def list(self, request, *args, **kwargs):
        """
        To get an actual value for object quotas limit and usage issue a **GET** request against */api/<objects>/*.

        To get all quotas visible to the user issue a **GET** request against */api/quotas/*
        """
        return super(QuotaViewSet, self).list(request, *args, **kwargs)

    def retrieve(self, request, *args, **kwargs):
        """
        To set quota limit issue a **PUT** request against */api/quotas/<quota uuid>** with limit values.

        Please note that if a quota is a cache of a backend quota (e.g. 'storage' size of an OpenStack tenant),
        it will be impossible to modify it through */api/quotas/<quota uuid>** endpoint.

        Example of changing quota limit:

        .. code-block:: http

            POST /api/quotas/6ad5f49d6d6c49648573b2b71f44a42b/ HTTP/1.1
            Content-Type: application/json
            Accept: application/json
            Authorization: Token c84d653b9ec92c6cbac41c706593e66f567a7fa4
            Host: example.com

            {
                "limit": 2000.0
            }

        Example of changing quota threshold:

        .. code-block:: http

            PUT /api/quotas/6ad5f49d6d6c49648573b2b71f44a42b/ HTTP/1.1
            Content-Type: application/json
            Accept: application/json
            Authorization: Token c84d653b9ec92c6cbac41c706593e66f567a7fa4
            Host: example.com

            {
                "threshold": 100.0
            }

        """
        return super(QuotaViewSet, self).retrieve(request, *args, **kwargs)

    def perform_update(self, serializer):
        if not serializer.instance.scope.can_user_update_quotas(self.request.user):
            raise rf_exceptions.PermissionDenied()
        quota = self.get_object()
        quota_field = quota.get_field()
        # old style quotas do not have quota_field
        if quota_field is not None and quota_field.is_backend:
            raise exceptions.BackendQuotaUpdateError()

        if 'limit' in serializer.validated_data:
            limit = serializer.validated_data['limit']
            if limit != -1 and quota.usage > limit:
                raise rf_exceptions.ValidationError(_('Current quota usage exceeds new limit.'))
            quota.scope.set_quota_limit(quota.name, limit)
            serializer.instance.refresh_from_db()

        if 'threshold' in serializer.validated_data:
            threshold = serializer.validated_data['threshold']
            quota.threshold = threshold
            quota.save(update_fields=['threshold'])
            serializer.instance.refresh_from_db()

    @decorators.detail_route()
    def history(self, request, uuid=None):
        """
        Historical data endpoints could be available for any objects (currently
        implemented for quotas and events count). The data is available at *<object_endpoint>/history/*,
        for example: */api/quotas/<uuid>/history/*.

        There are two ways to define datetime points for historical data.

        1. Send *?point=<timestamp>* parameter that can list. Response will contain historical data for each given point
            in the same order.
        2. Send *?start=<timestamp>*, *?end=<timestamp>*, *?points_count=<integer>* parameters.
           Result will contain <points_count> points from <start> to <end>.

        Response format:

        .. code-block:: javascript

            [
                {
                    "point": <timestamp>,
                    "object": {<object_representation>}
                },
                {
                    "point": <timestamp>
                    "object": {<object_representation>}
                },
            ...
            ]

        NB! There will not be any "object" for corresponding point in response if there
        is no data about object for a given timestamp.
        """
        mapped = {
            'start': request.query_params.get('start'),
            'end': request.query_params.get('end'),
            'points_count': request.query_params.get('points_count'),
            'point_list': request.query_params.getlist('point'),
        }
        history_serializer = HistorySerializer(data={k: v for k, v in mapped.items() if v})
        history_serializer.is_valid(raise_exception=True)

        quota = self.get_object()
        serializer = self.get_serializer(quota)
        serialized_versions = []
        for point_date in history_serializer.get_filter_data():
            serialized = {'point': datetime_to_timestamp(point_date)}
            version = Version.objects.get_for_object(quota).filter(revision__date_created__lte=point_date)
            if version.exists():
                # make copy of serialized data and update field that are stored in version
                version_object = version.first()._object_version.object
                serialized['object'] = serializer.data.copy()
                serialized['object'].update({
                    f: getattr(version_object, f) for f in quota.get_version_fields()
                })
            serialized_versions.append(serialized)
        return response.Response(serialized_versions, status=status.HTTP_200_OK)



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.contrib.contenttypes import models as ct_models
from django.db import migrations, models

from waldur_core.quotas.models import Quota
from waldur_core.structure.models import CloudServiceProjectLink


def reset_cloud_spl_quota_limits(apps, schema_editor):
    old_limits = {
        'vcpu': 100,
        'ram': 256000,
        'storage': 5120000,
    }

    for model in CloudServiceProjectLink.get_all_models():
        content_type = ct_models.ContentType.objects.get_for_model(model)
        for quota, limit in old_limits.items():
            Quota.objects.filter(content_type=content_type, name=quota, limit=limit).update(limit=-1)


class Migration(migrations.Migration):

    dependencies = [
        ('structure', '0049_extend_abbreviation'),
    ]

    operations = [
        migrations.RunPython(reset_cloud_spl_quota_limits),
    ]



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations
import waldur_core.core.fields
import django.core.validators

import waldur_core.core.validators


class Migration(migrations.Migration):

    replaces = [('cost_tracking', '0001_initial'), ('cost_tracking', '0002_price_list'), ('cost_tracking', '0003_new_price_list_items'), ('cost_tracking', '0004_remove_connection_to_resource'), ('cost_tracking', '0005_expand_item_type_size'), ('cost_tracking', '0006_add_backend_cache_fields_to_pricelist'), ('cost_tracking', '0007_remove_obsolete_billing_fields'), ('cost_tracking', '0008_delete_resourceusage'), ('cost_tracking', '0009_defaultpricelistitem_name'), ('cost_tracking', '0010_applicationtype'), ('cost_tracking', '0011_applicationtype_slug'), ('cost_tracking', '0012_fix_fields_length')]

    dependencies = [
        ('contenttypes', '0001_initial'),
    ]

    operations = [
        migrations.CreateModel(
            name='PriceEstimate',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('uuid', waldur_core.core.fields.UUIDField()),
                ('object_id', models.PositiveIntegerField()),
                ('total', models.FloatField(default=0)),
                ('details', waldur_core.core.fields.JSONField(blank=True)),
                ('month', models.PositiveSmallIntegerField(validators=[django.core.validators.MaxValueValidator(12), django.core.validators.MinValueValidator(1)])),
                ('year', models.PositiveSmallIntegerField()),
                ('is_manually_input', models.BooleanField(default=False)),
                ('is_visible', models.BooleanField(default=True)),
                ('content_type', models.ForeignKey(to='contenttypes.ContentType')),
            ],
            options={
                'unique_together': set([('content_type', 'object_id', 'month', 'year', 'is_manually_input')]),
            },
            bases=(models.Model,),
        ),
        migrations.CreateModel(
            name='PriceListItem',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('uuid', waldur_core.core.fields.UUIDField()),
                ('content_type', models.ForeignKey(to='contenttypes.ContentType')),
                ('object_id', models.PositiveIntegerField()),
                ('key', models.CharField(max_length=255)),
                ('value', models.DecimalField(default=0, verbose_name=b'Hourly rate', max_digits=9, decimal_places=2)),
                ('units', models.CharField(max_length=255, blank=True)),
                ('item_type', models.CharField(default=b'flavor', max_length=255, choices=[(b'flavor', b'flavor'), (b'storage', b'storage'), (b'license-application', b'license-application'), (b'license-os', b'license-os'), (b'support', b'support'), (b'network', b'network'), (b'usage', b'usage'), (b'users', b'users')])),
                ('is_manually_input', models.BooleanField(default=False)),
                ('resource_content_type', models.ForeignKey(related_name='+', default=None, to='contenttypes.ContentType')),
            ],
            options={
                'unique_together': set([('key', 'content_type', 'object_id')]),
            },
            bases=(models.Model,),
        ),
        migrations.CreateModel(
            name='DefaultPriceListItem',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('name', models.CharField(max_length=150, verbose_name='name', validators=[waldur_core.core.validators.validate_name])),
                ('uuid', waldur_core.core.fields.UUIDField()),
                ('key', models.CharField(max_length=255)),
                ('value', models.DecimalField(default=0, verbose_name=b'Hourly rate', max_digits=9, decimal_places=2)),
                ('units', models.CharField(max_length=255, blank=True)),
                ('item_type', models.CharField(default=b'flavor', max_length=255, choices=[(b'flavor', b'flavor'), (b'storage', b'storage'), (b'license-application', b'license-application'), (b'license-os', b'license-os'), (b'support', b'support'), (b'network', b'network'), (b'usage', b'usage'), (b'users', b'users')])),
                ('resource_content_type', models.ForeignKey(default=None, to='contenttypes.ContentType')),
            ],
            options={
                'abstract': False,
            },
            bases=(models.Model,),
        ),
        migrations.CreateModel(
            name='ApplicationType',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('name', models.CharField(max_length=150, verbose_name='name', validators=[waldur_core.core.validators.validate_name])),
                ('slug', models.CharField(unique=True, max_length=150)),
            ],
            options={
                'abstract': False,
            },
            bases=(models.Model,),
        ),
    ]



from django.conf.urls import url

from waldur_core.logging import views


def register_in(router):
    router.register(r'alerts', views.AlertViewSet)
    router.register(r'events', views.EventViewSet, base_name='event')
    router.register(r'hooks-web', views.WebHookViewSet, base_name='webhook')
    router.register(r'hooks-push', views.PushHookViewSet, base_name='pushhook')
    router.register(r'hooks-email', views.EmailHookViewSet, base_name='emailhook')
    router.register(r'hooks', views.HookSummary, base_name='hooks')


events_count_history = views.EventViewSet.as_view({'get': 'count_history'})
stats_alerts = views.AlertViewSet.as_view({'get': 'stats'})

urlpatterns = [
    # Separate history URL for consistency with other history endpoints
    url(r'^events/count/history/', events_count_history, name='event-count-history'),
    # Hook for backward compatibility with old URL
    url(r'^stats/alerts/', stats_alerts, name='stats_alerts'),
]



import inspect
import pkg_resources

default_app_config = 'waldur_core.core.apps.CoreConfig'


class WaldurExtension(object):
    """ Base class for Waldur extensions """

    class Settings:
        """ Defines extra django settings """
        pass

    @staticmethod
    def update_settings(settings):
        pass

    @staticmethod
    def django_app():
        """ Returns a django application name which will be added to INSTALLED_APPS """
        raise NotImplementedError

    @staticmethod
    def django_urls():
        """ Returns a list of django URL in urlpatterns format """
        return []

    @staticmethod
    def rest_urls():
        """ Returns a function which register URLs in REST API """
        return lambda router: NotImplemented

    @staticmethod
    def celery_tasks():
        """ Returns a dictionary with celery tasks which will be added to CELERYBEAT_SCHEDULE """
        return dict()

    @staticmethod
    def get_cleanup_executor():
        """ Returns a Celery task to cleanup project resources """
        pass

    @staticmethod
    def is_assembly():
        """ Return True if plugin is assembly and should be installed last """
        return False

    @classmethod
    def get_extensions(cls):
        """ Get a list of available extensions """
        assemblies = []
        for waldur_extension in pkg_resources.iter_entry_points('waldur_extensions'):
            extension_module = waldur_extension.load()
            if inspect.isclass(extension_module) and issubclass(extension_module, cls):
                if not extension_module.is_assembly():
                    yield extension_module
                else:
                    assemblies.append(extension_module)
        for assembly in assemblies:
            yield assembly

    @classmethod
    def is_installed(cls, extension):
        for ext in cls.get_extensions():
            if extension == ext.django_app():
                return True
        return False



from datetime import timedelta
import mock

from django.contrib.auth import get_user_model
from django.contrib.contenttypes.models import ContentType
from django.db import IntegrityError
from django.utils import timezone
from rest_framework import test, status

from waldur_core.core import utils as core_utils
from waldur_core.logging import models, loggers
from waldur_core.logging.tests import factories
# Dependency from `structure` application exists only in tests
from waldur_core.structure import models as structure_models
from waldur_core.structure.tests import factories as structure_factories


class AlertsListTest(test.APITransactionTestCase):

    def setUp(self):
        self.customer = structure_factories.CustomerFactory()
        self.owner = structure_factories.UserFactory()
        self.customer.add_user(self.owner, structure_models.CustomerRole.OWNER)

    def test_customer_owner_can_see_alert_about_his_project(self):
        project = structure_factories.ProjectFactory(customer=self.customer)
        alert = factories.AlertFactory(scope=project)

        self.client.force_authenticate(self.owner)
        response = self.client.get(factories.AlertFactory.get_list_url())

        self.assertEqual(response.status_code, status.HTTP_200_OK)
        self.assertIn(alert.uuid.hex, [a['uuid'] for a in response.data])

    def test_customer_owner_cannot_see_alert_about_other_customer(self):
        alert = factories.AlertFactory()

        self.client.force_authenticate(self.owner)
        response = self.client.get(factories.AlertFactory.get_list_url())

        self.assertEqual(response.status_code, status.HTTP_200_OK)
        self.assertNotIn(alert.uuid.hex, [a['uuid'] for a in response.data])

    def test_alert_list_can_be_filtered_by_scope(self):
        project = structure_factories.ProjectFactory(customer=self.customer)
        alert1 = factories.AlertFactory(scope=project)
        alert2 = factories.AlertFactory()

        self.client.force_authenticate(self.owner)
        response = self.client.get(factories.AlertFactory.get_list_url(), data={
            'scope': structure_factories.ProjectFactory.get_url(project)})

        self.assertEqual(response.status_code, status.HTTP_200_OK)
        self.assertIn(alert1.uuid.hex, [a['uuid'] for a in response.data])
        self.assertNotIn(alert2.uuid.hex, [a['uuid'] for a in response.data])

    def test_alert_list_can_be_filtered_by_scope_type(self):
        # XXX: this tests will removed after content type filter implementation at portal
        project = structure_factories.ProjectFactory(customer=self.customer)
        alert1 = factories.AlertFactory(scope=project)
        alert2 = factories.AlertFactory(scope=self.customer)

        self.client.force_authenticate(self.owner)
        response = self.client.get(factories.AlertFactory.get_list_url(), data={'scope_type': 'customer'})

        self.assertEqual(response.status_code, status.HTTP_200_OK)
        self.assertIn(alert2.uuid.hex, [a['uuid'] for a in response.data])
        self.assertNotIn(alert1.uuid.hex, [a['uuid'] for a in response.data])

    def test_alert_list_can_be_aggregated_for_concreate_customer(self):
        project = structure_factories.ProjectFactory(customer=self.customer)
        alert1 = factories.AlertFactory(scope=project)
        alert2 = factories.AlertFactory()

        self.client.force_authenticate(self.owner)
        response = self.client.get(factories.AlertFactory.get_list_url(), data={
            'aggregate': 'customer', 'uuid': self.customer.uuid.hex})

        self.assertEqual(response.status_code, status.HTTP_200_OK)
        self.assertIn(alert1.uuid.hex, [a['uuid'] for a in response.data])
        self.assertNotIn(alert2.uuid.hex, [a['uuid'] for a in response.data])

    def test_alert_list_can_be_filtered_by_created_date(self):
        project = structure_factories.ProjectFactory(customer=self.customer)
        alert1 = factories.AlertFactory(scope=project, created=timezone.now()-timedelta(days=1))
        alert2 = factories.AlertFactory(scope=project, created=timezone.now()-timedelta(days=3))

        self.client.force_authenticate(self.owner)
        response = self.client.get(factories.AlertFactory.get_list_url(), data={
            'created_from': core_utils.datetime_to_timestamp(timezone.now()-timedelta(days=2)),
            'created_to': core_utils.datetime_to_timestamp(timezone.now())})

        self.assertEqual(response.status_code, status.HTTP_200_OK)
        self.assertIn(alert1.uuid.hex, [a['uuid'] for a in response.data])
        self.assertNotIn(alert2.uuid.hex, [a['uuid'] for a in response.data])

    def test_alert_list_can_be_filtered_by_severity_list(self):
        project = structure_factories.ProjectFactory(customer=self.customer)
        alert1 = factories.AlertFactory(scope=project, severity=models.Alert.SeverityChoices.WARNING)
        alert2 = factories.AlertFactory(scope=project, severity=models.Alert.SeverityChoices.ERROR)
        alert3 = factories.AlertFactory(scope=project, severity=models.Alert.SeverityChoices.INFO)

        self.client.force_authenticate(self.owner)
        response = self.client.get(factories.AlertFactory.get_list_url(), data={'severity': ['Warning', 'Error']})

        self.assertEqual(response.status_code, status.HTTP_200_OK)
        self.assertIn(alert1.uuid.hex, [a['uuid'] for a in response.data])
        self.assertIn(alert2.uuid.hex, [a['uuid'] for a in response.data])
        self.assertNotIn(alert3.uuid.hex, [a['uuid'] for a in response.data])

    def test_alert_can_be_filtered_by_content_type(self):
        project = structure_factories.ProjectFactory(customer=self.customer)
        alert1 = factories.AlertFactory(scope=project)
        alert2 = factories.AlertFactory(scope=self.customer)

        self.client.force_authenticate(self.owner)
        response = self.client.get(factories.AlertFactory.get_list_url(), data={'content_type': ['structure.project']})

        self.assertEqual(response.status_code, status.HTTP_200_OK)
        self.assertIn(alert1.uuid.hex, [a['uuid'] for a in response.data])
        self.assertNotIn(alert2.uuid.hex, [a['uuid'] for a in response.data])

    def test_alert_can_be_filtered_by_scope(self):
        project = structure_factories.ProjectFactory(customer=self.customer)
        alert1 = factories.AlertFactory(scope=project)
        alert2 = factories.AlertFactory(scope=self.customer)

        self.client.force_authenticate(self.owner)
        response = self.client.get(
            factories.AlertFactory.get_list_url(),
            data={'scope': structure_factories.ProjectFactory.get_url(project)})

        self.assertEqual(response.status_code, status.HTTP_200_OK)
        self.assertIn(alert1.uuid.hex, [a['uuid'] for a in response.data])
        self.assertNotIn(alert2.uuid.hex, [a['uuid'] for a in response.data])


class AlertsCreateUpdateDeleteTest(test.APITransactionTestCase):

    def setUp(self):
        self.project = structure_factories.ProjectFactory()
        self.staff = get_user_model().objects.create_superuser(
            username='staff', password='staff', email='staff@example.com')
        self.alert = factories.AlertFactory.build(scope=self.project, severity=10)
        severity_names = dict(models.Alert.SeverityChoices.CHOICES)
        self.valid_data = {
            'scope': structure_factories.ProjectFactory.get_url(self.project),
            'alert_type': self.alert.alert_type,
            'message': self.alert.message,
            'severity': severity_names[self.alert.severity],
        }
        self.url = factories.AlertFactory.get_list_url()

    def test_alert_can_be_created_by_staff(self):
        self.client.force_authenticate(self.staff)
        response = self.client.post(self.url, data=self.valid_data)

        self.assertEqual(response.status_code, status.HTTP_201_CREATED)
        ct = ContentType.objects.get_for_model(structure_models.Project)
        self.assertTrue(models.Alert.objects.filter(
            content_type=ct, object_id=self.project.id, alert_type=self.alert.alert_type).exists())

    def test_alert_severity_can_be_updated(self):
        self.alert.save()
        self.valid_data['severity'] = 'Error'

        self.client.force_authenticate(self.staff)
        response = self.client.post(self.url, data=self.valid_data)

        self.assertEqual(response.status_code, status.HTTP_201_CREATED)
        ct = ContentType.objects.get_for_model(structure_models.Project)
        self.assertTrue(models.Alert.objects.filter(
            pk=self.alert.pk,
            content_type=ct,
            object_id=self.project.id,
            alert_type=self.alert.alert_type,
            severity=models.Alert.SeverityChoices.ERROR).exists()
        )

    def test_alert_can_be_closed(self):
        self.alert.save()

        self.client.force_authenticate(self.staff)
        response = self.client.post(factories.AlertFactory.get_url(self.alert, 'close'))

        self.assertEqual(response.status_code, status.HTTP_204_NO_CONTENT)
        ct = ContentType.objects.get_for_model(structure_models.Project)
        self.assertTrue(models.Alert.objects.filter(
            content_type=ct,
            object_id=self.project.id,
            alert_type=self.alert.alert_type,
            closed__isnull=False).exists()
        )


class TestAlertActions(test.APITransactionTestCase):

    def setUp(self):
        self.project = structure_factories.ProjectFactory()
        self.staff = get_user_model().objects.create_superuser(
            username='staff', password='staff', email='staff@example.com')
        self.alert = factories.AlertFactory(scope=self.project)
        self.admin = structure_factories.UserFactory()
        self.project.add_user(self.admin, structure_models.ProjectRole.ADMINISTRATOR)

    def test_alert_can_be_closed_by_staff(self):
        self.client.force_authenticate(self.staff)
        response = self.client.post(factories.AlertFactory.get_url(self.alert, 'close'))

        self.assertEqual(response.status_code, status.HTTP_204_NO_CONTENT)
        reread_alert = models.Alert.objects.get(pk=self.alert.pk)
        self.assertTrue(reread_alert.closed is not None)

    def test_alert_can_not_be_closed_by_project_administrator(self):
        self.client.force_authenticate(self.admin)
        response = self.client.post(factories.AlertFactory.get_url(self.alert, 'close'))

        self.assertEqual(response.status_code, status.HTTP_403_FORBIDDEN)
        reread_alert = models.Alert.objects.get(pk=self.alert.pk)
        self.assertTrue(reread_alert.closed is None)

    def test_alert_can_be_marked_as_acknowledged_by_project_administrator(self):
        self.client.force_authenticate(self.admin)
        response = self.client.post(factories.AlertFactory.get_url(self.alert, 'acknowledge'))

        self.assertEqual(response.status_code, status.HTTP_200_OK)
        reread_alert = models.Alert.objects.get(pk=self.alert.pk)
        self.assertTrue(reread_alert.acknowledged)

    def test_acknowledged_alert_cannot_be_marked_as_acknowledged_again(self):
        self.alert.acknowledge()

        self.client.force_authenticate(self.admin)
        response = self.client.post(factories.AlertFactory.get_url(self.alert, 'acknowledge'))

        self.assertEqual(response.status_code, status.HTTP_409_CONFLICT)

    def test_admin_can_cancel_acknowledgment_of_acknowledged_alert(self):
        self.alert.acknowledge()

        self.client.force_authenticate(self.admin)
        response = self.client.post(factories.AlertFactory.get_url(self.alert, 'cancel_acknowledgment'))

        self.assertEqual(response.status_code, status.HTTP_200_OK)
        reread_alert = models.Alert.objects.get(pk=self.alert.pk)
        self.assertFalse(reread_alert.acknowledged)


class AlertUniquenessTest(test.APITransactionTestCase):

    def setUp(self):
        self.project = structure_factories.ProjectFactory()

    def get_logger(self):
        if not hasattr(loggers.alert_logger, 'test_alert_logger'):
            class TestAlertLogger(loggers.AlertLogger):
                class Meta:
                    alert_types = ('test_alert',)

            loggers.alert_logger.register('test_alert_logger', TestAlertLogger)

        return loggers.alert_logger.test_alert_logger

    def log_alert(self):
        return self.get_logger().info('Message', scope=self.project, alert_type='test_alert')

    def test_duplicate_alert_is_not_created(self):
        alert, created = self.log_alert()
        self.assertEqual(created, True)

        alert, created = self.log_alert()
        self.assertEqual(created, False)

    def test_if_race_conditions_detected_alert_skipped(self):
        with mock.patch('waldur_core.logging.loggers.models') as mock_models:
            mock_models.Alert.objects.create.side_effect = IntegrityError

            alert, created = self.log_alert()
            self.assertEqual(created, False)



import logging

from django.conf import settings as django_settings
from rest_framework import exceptions

from waldur_core.core.permissions import SAFE_METHODS, IsAdminOrReadOnly
from waldur_core.structure import models

logger = logging.getLogger(__name__)


def _can_manage_organization(candidate_user, approving_user):
    if candidate_user.organization == "":
        return False

    # TODO: this will fail validation if more than one customer with a particular abbreviation exists
    try:
        organization = models.Customer.objects.get(abbreviation=candidate_user.organization)
        if organization.has_user(approving_user):
            return True
    except models.Customer.DoesNotExist:
        logger.warning('Approval was attempted for a customer with abbreviation %s that does not exist.',
                       candidate_user.organization)
    except models.Customer.MultipleObjectsReturned:
        logger.error('More than one customer with abbreviation %s exists. Breaks approval flow.',
                     candidate_user.organization)

    return False


# TODO: this is a temporary permission filter.
class IsAdminOrOwnerOrOrganizationManager(IsAdminOrReadOnly):
    """
    Allows access to admin users or account's owner for modifications.
    Allow access for approving/rejecting/removing organization for connected customer owners.
    For other users read-only access.
    """

    def has_permission(self, request, view):
        approving_user = request.user
        if approving_user.is_staff or request.method in SAFE_METHODS:
            return True
        elif view.suffix == 'List' or request.method == 'DELETE':
            return False
        # Fix for schema generation
        elif 'uuid' not in view.kwargs:
            return False
        elif request.method == 'POST' and view.action_map.get('post') in \
                ['approve_organization', 'reject_organization', 'remove_organization']:

            candidate_user = view.get_object()
            if approving_user == candidate_user and view.action_map.get('post') == 'remove_organization' \
                    and not candidate_user.organization_approved:
                return True

            return _can_manage_organization(candidate_user, approving_user)

        return approving_user == view.get_object()


def is_staff(request, view, obj=None):
    if not request.user.is_staff:
        raise exceptions.PermissionDenied()


def is_owner(request, view, obj=None):
    if not obj:
        return
    customer = _get_customer(obj)
    if not _has_owner_access(request.user, customer):
        raise exceptions.PermissionDenied()


def is_manager(request, view, obj=None):
    if not obj:
        return
    project = _get_project(obj)
    if not _has_manager_access(request.user, project):
        raise exceptions.PermissionDenied()


def is_administrator(request, view, obj=None):
    if not obj:
        return
    project = _get_project(obj)
    if not _has_admin_access(request.user, project):
        raise exceptions.PermissionDenied()


def _has_owner_access(user, customer):
    return user.is_staff or customer.has_user(user, models.CustomerRole.OWNER)


def _has_manager_access(user, project):
    return _has_owner_access(user, project.customer) or project.has_user(user, models.ProjectRole.MANAGER)


def _has_admin_access(user, project):
    return _has_manager_access(user, project) or project.has_user(user, models.ProjectRole.ADMINISTRATOR)


def _get_parent_by_permission_path(obj, permission_path):
    path = getattr(obj.Permissions, permission_path, None)
    if path is None:
        return
    if path == 'self':
        return obj
    return reduce(getattr, path.split('__'), obj)


def _get_project(obj):
    return _get_parent_by_permission_path(obj, 'project_path')


def _get_customer(obj):
    return _get_parent_by_permission_path(obj, 'customer_path')


def check_access_to_services_management(request, view, obj=None):
    if django_settings.WALDUR_CORE['ONLY_STAFF_MANAGES_SERVICES'] and not request.user.is_staff:
        raise exceptions.PermissionDenied()



from __future__ import unicode_literals

from django.utils.translation import ugettext_lazy as _
from rest_framework import status
from rest_framework.exceptions import APIException


class IncorrectStateException(APIException):
    status_code = status.HTTP_409_CONFLICT
    default_detail = _('Cannot modify an object in its current state.')



from django.apps import AppConfig

from waldur_core.structure import SupportedServices, ServiceBackend


default_app_config = 'waldur_core.structure.tests.TestConfig'


class TestBackend(ServiceBackend):
    def destroy(self, resource, force=False):
        pass


class TestConfig(AppConfig):
    name = 'waldur_core.structure.tests'
    label = 'structure_tests'
    service_name = 'Test'

    def ready(self):
        SupportedServices.register_backend(TestBackend)
        SupportedServices.register_service(self.get_model('TestService'))



from mock import patch

from django.test import TestCase

from .. import factories


class LogProjectSaveTest(TestCase):

    @patch('waldur_core.structure.handlers.event_logger')
    def test_logger_called_once_on_project_create(self, logger_mock):
        new_project = factories.ProjectFactory()

        logger_mock.project.info.assert_called_once_with(
            'Project {project_name} has been created.',
            event_type='project_creation_succeeded',
            event_context={
                'project': new_project,
            },
        )

    def test_logger_called_once_on_project_name_update(self):
        new_project = factories.ProjectFactory()
        old_name = new_project.name

        with patch('waldur_core.structure.handlers.event_logger') as logger_mock:
            new_project.name = 'new name'
            new_project.save()

            logger_mock.project.info.assert_called_once_with(
                "Project {project_name} has been updated. Name has been changed from '%s' to '%s'." % (
                    old_name,
                    new_project.name,
                ),
                event_type='project_update_succeeded',
                event_context={
                    'project': new_project,
                },
            )

    def test_logger_logs_project_name_and_description_when_updated(self):
        new_project = factories.ProjectFactory(description='description', name='name')

        with patch('waldur_core.structure.handlers.event_logger') as logger_mock:
            new_project.name = 'new name'
            new_project.description = 'new description'
            new_project.save()

            expected_message = ('Project {project_name} has been updated.'
                                " Description has been changed from 'description' to 'new description'."
                                " Name has been changed from 'name' to 'new name'.")
            logger_mock.project.info.assert_called_once_with(
                expected_message,
                event_type='project_update_succeeded',
                event_context={
                    'project': new_project,
                },
            )




from __future__ import unicode_literals

import logging

from smtplib import SMTPException

from celery import shared_task
from django.conf import settings
from django.core.mail import send_mail
from django.template.loader import render_to_string
from django.utils import timezone

from waldur_core.structure.models import ProjectRole
from waldur_core.users import models


logger = logging.getLogger(__name__)


@shared_task(name='waldur_core.users.cancel_expired_invitations')
def cancel_expired_invitations(invitations=None):
    """
    Invitation lifetime must be specified in Waldur Core settings with parameter
    "INVITATION_LIFETIME". If invitation creation time is less than expiration time, the invitation will set as expired.
    """
    expiration_date = timezone.now() - settings.WALDUR_CORE['INVITATION_LIFETIME']
    if not invitations:
        invitations = models.Invitation.objects.filter(state=models.Invitation.State.PENDING)
    invitations = invitations.filter(created__lte=expiration_date)
    invitations.update(state=models.Invitation.State.EXPIRED)


@shared_task(name='waldur_core.users.send_invitation')
def send_invitation(invitation_uuid, sender_name):
    invitation = models.Invitation.objects.get(uuid=invitation_uuid)

    if invitation.project_role is not None:
        context = dict(type='project', name=invitation.project.name)
        role_prefix = 'project' if invitation.project_role == ProjectRole.MANAGER else 'system'
        context['role'] = '%s %s' % (role_prefix, invitation.get_project_role_display())

    else:
        context = dict(
            type='organization',
            name=invitation.customer.name,
            role=invitation.get_customer_role_display()
        )

    context['sender'] = sender_name
    context['link'] = invitation.link_template.format(uuid=invitation_uuid)

    subject = render_to_string('users/invitation_subject.txt', context)
    text_message = render_to_string('users/invitation_message.txt', context)
    html_message = render_to_string('users/invitation_message.html', context)

    logger.debug('About to send invitation to {email} to join {name} {type} as {role}'.format(
        email=invitation.email, **context))
    try:
        send_mail(subject, text_message, settings.DEFAULT_FROM_EMAIL, [invitation.email], html_message=html_message)
    except SMTPException as e:
        invitation.error_message = str(e)
        invitation.save(update_fields=['error_message'])
        raise



from ddt import ddt, data
from django.urls import reverse
from rest_framework import status, test

from waldur_core.logging import loggers
from waldur_core.logging.tests.factories import WebHookFactory, PushHookFactory
from waldur_core.structure.tests import factories as structure_factories


class BaseHookApiTest(test.APITransactionTestCase):
    def setUp(self):
        self.staff = structure_factories.UserFactory(is_staff=True)
        self.author = structure_factories.UserFactory()
        self.other_user = structure_factories.UserFactory()

        self.valid_event_types = loggers.get_valid_events()[:3]
        self.valid_event_groups = loggers.get_event_groups_keys()


class HookCreationViewTest(BaseHookApiTest):

    def test_user_can_create_webhook(self):
        self.client.force_authenticate(user=self.author)
        response = self.client.post(WebHookFactory.get_list_url(), data={
            'event_types': self.valid_event_types,
            'destination_url': 'http://example.com/'
        })
        self.assertEqual(response.status_code, status.HTTP_201_CREATED, response.data)

    def test_user_can_create_email_hook(self):
        self.client.force_authenticate(user=self.author)
        response = self.client.post(reverse('emailhook-list'), data={
            'event_types': self.valid_event_types,
            'email': 'test@example.com'
        })
        self.assertEqual(response.status_code, status.HTTP_201_CREATED, response.data)

    def test_user_can_create_push_hook(self):
        self.client.force_authenticate(user=self.author)
        response = self.client.post(PushHookFactory.get_list_url(), data={
            'event_types': self.valid_event_types,
            'token': 'VALID_TOKEN',
            'type': 'Android'
        })
        self.assertEqual(response.status_code, status.HTTP_201_CREATED, response.data)

    def test_user_can_subscribe_to_event_groups(self):
        event_groups = self.valid_event_groups
        event_types = loggers.expand_event_groups(event_groups)

        self.client.force_authenticate(user=self.author)
        response = self.client.post(WebHookFactory.get_list_url(), data={
            'event_groups': event_groups,
            'destination_url': 'http://example.com/'
        })
        self.assertEqual(response.status_code, status.HTTP_201_CREATED)
        self.assertEqual(response.data['event_groups'], set(event_groups))
        self.assertEqual(response.data['event_types'], set(event_types))


@ddt
class HookUpdateTest(BaseHookApiTest):
    def setUp(self):
        super(HookUpdateTest, self).setUp()
        self.hooks = {
            'web': WebHookFactory.get_url(WebHookFactory(user=self.author)),
            'push': PushHookFactory.get_url(PushHookFactory(user=self.author))
        }

    def test_author_can_update_webhook_destination_url(self):
        new_data = {
            'destination_url': 'http://another-host.com'
        }
        response = self.update_hook('web', new_data)
        self.assertEqual(new_data['destination_url'], response.data['destination_url'])

    def test_author_can_update_push_hook_token(self):
        new_data = {
            'token': 'NEW_VALID_TOKEN'
        }
        response = self.update_hook('push', new_data)
        self.assertEqual(new_data['token'], response.data['token'])

    @data('web', 'push')
    def test_author_can_update_hook_event_types(self, hook):
        new_event_types = set(self.valid_event_types[:1])
        response = self.update_hook(hook, {'event_types': new_event_types})
        self.assertEqual(new_event_types, response.data['event_types'])

    @data('web', 'push')
    def test_author_can_update_event_groups(self, hook):
        event_groups = self.valid_event_groups
        event_types = loggers.expand_event_groups(event_groups)

        self.client.force_authenticate(user=self.author)
        response = self.update_hook(hook, {
            'event_groups': event_groups
        })
        self.assertEqual(response.data['event_groups'], set(event_groups))
        self.assertEqual(response.data['event_types'], set(event_types))

    @data('web', 'push')
    def test_author_can_disable_hook(self, hook):
        response = self.update_hook(hook, {'is_active': False})
        self.assertFalse(response.data['is_active'])

    def update_hook(self, hook, data):
        self.client.force_authenticate(user=self.author)
        url = self.hooks[hook]
        response = self.client.patch(url, data=data)
        self.assertEqual(response.status_code, status.HTTP_200_OK, response.data)
        return response


class HookPermissionsViewTest(BaseHookApiTest):

    def setUp(self):
        super(HookPermissionsViewTest, self).setUp()
        self.url = WebHookFactory.get_url(WebHookFactory(user=self.author))

    def test_hook_visible_to_author(self):
        self.client.force_authenticate(user=self.author)
        response = self.client.get(self.url)
        self.assertEqual(response.status_code, status.HTTP_200_OK, response.data)
        self.assertEqual(self.author.uuid, response.data['author_uuid'])

    def test_hook_visible_to_staff(self):
        self.client.force_authenticate(user=self.staff)
        response = self.client.get(self.url)
        self.assertEqual(response.status_code, status.HTTP_200_OK, response.data)

    def test_hook_not_visible_to_other_user(self):
        self.client.force_authenticate(user=self.other_user)
        response = self.client.get(self.url)
        self.assertEqual(response.status_code, status.HTTP_404_NOT_FOUND, response.data)


class HookFilterViewTest(BaseHookApiTest):
    def test_staff_can_filter_webhook_by_author_uuid(self):
        WebHookFactory(user=self.author)
        WebHookFactory(user=self.other_user)
        self.client.force_authenticate(user=self.staff)
        response = self.client.get(WebHookFactory.get_list_url(), {'author_uuid': self.author.uuid.hex})
        self.assertEqual(response.status_code, status.HTTP_200_OK, response.data)
        self.assertEqual(len(response.data), 1)
        self.assertEqual(self.author.uuid, response.data[0]['author_uuid'])

    def test_staff_can_filter_summary_hook_by_author_uuid(self):
        WebHookFactory(user=self.author)
        PushHookFactory(user=self.other_user)
        self.client.force_authenticate(user=self.staff)
        response = self.client.get(reverse('hooks-list'), {'author_uuid': self.author.uuid.hex})
        self.assertEqual(response.status_code, status.HTTP_200_OK, response.data)
        self.assertEqual(len(response.data), 1)
        self.assertEqual(self.author.uuid, response.data[0]['author_uuid'])



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import migrations, models
import waldur_core.core.fields
import waldur_core.core.validators


class Migration(migrations.Migration):

    dependencies = [
        ('structure', '0041_servicesettings_domain'),
    ]

    operations = [
        migrations.CreateModel(
            name='ServiceCertification',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('description', models.CharField(max_length=500, verbose_name='description', blank=True)),
                ('name', models.CharField(max_length=150, verbose_name='name', validators=[waldur_core.core.validators.validate_name])),
                ('uuid', waldur_core.core.fields.UUIDField()),
                ('link', models.URLField(max_length=255, blank=True)),
            ],
            options={
                'ordering': ['-name'],
                'verbose_name': 'Service Certification',
                'verbose_name_plural': 'Service Certifications',
            },
        ),
        migrations.AddField(
            model_name='servicesettings',
            name='homepage',
            field=models.URLField(max_length=255, blank=True),
        ),
        migrations.AddField(
            model_name='servicesettings',
            name='terms_of_services',
            field=models.TextField(blank=True),
        ),
        migrations.AddField(
            model_name='servicesettings',
            name='certifications',
            field=models.ManyToManyField(blank=True, related_name='service_settings', to='structure.ServiceCertification'),
        ),
    ]



from django.conf import settings
from django.contrib.auth import get_user_model
from django.urls import reverse
from django.utils import timezone
from django.core.cache import cache

from freezegun import freeze_time

from rest_framework import test, status
from rest_framework.authtoken.models import Token


class TokenAuthenticationTest(test.APITransactionTestCase):
    def setUp(self):
        self.username = 'test'
        self.password = 'secret'
        self.auth_url = 'http://testserver' + reverse('auth-password')
        self.test_url = 'http://testserver/api/'
        get_user_model().objects.create_user(self.username, 'admin@example.com', self.password)

    def tearDown(self):
        cache.clear()

    def test_user_can_authenticate_with_token(self):
        response = self.client.post(self.auth_url, data={'username': self.username, 'password': self.password})
        self.assertEqual(response.status_code, status.HTTP_200_OK)

        token = response.data['token']
        self.client.credentials(HTTP_AUTHORIZATION='Token ' + token)
        response = self.client.get(self.test_url)
        self.assertEqual(response.status_code, status.HTTP_200_OK)

    def test_token_expires_based_on_user_token_lifetime(self):
        user = get_user_model().objects.get(username=self.username)
        configured_token_lifetime = settings.WALDUR_CORE.get('TOKEN_LIFETIME', timezone.timedelta(hours=1))
        user_token_lifetime = configured_token_lifetime - timezone.timedelta(seconds=40)
        user.token_lifetime = user_token_lifetime.seconds
        user.save()

        response = self.client.post(self.auth_url, data={'username': self.username, 'password': self.password})
        self.assertEqual(response.status_code, status.HTTP_200_OK)

        token = response.data['token']
        mocked_now = timezone.now() + user_token_lifetime
        with freeze_time(mocked_now):
            self.client.credentials(HTTP_AUTHORIZATION='Token ' + token)
            response = self.client.get(self.test_url)
            self.assertEqual(response.status_code, status.HTTP_401_UNAUTHORIZED)
            self.assertEqual(response.data['detail'], 'Token has expired.')

    def test_token_creation_time_is_updated_on_every_request(self):
        response = self.client.post(self.auth_url, data={'username': self.username, 'password': self.password})
        self.assertEqual(response.status_code, status.HTTP_200_OK)
        token = response.data['token']
        created1 = Token.objects.values_list('created', flat=True).get(key=token)

        self.client.credentials(HTTP_AUTHORIZATION='Token ' + token)
        self.client.get(self.test_url)
        created2 = Token.objects.values_list('created', flat=True).get(key=token)
        self.assertTrue(created1 < created2)

    def test_account_is_blocked_after_five_failed_attempts(self):
        for _ in range(5):
            response = self.client.post(self.auth_url, data={'username': self.username, 'password': 'WRONG'})
            self.assertEqual(response.status_code, status.HTTP_401_UNAUTHORIZED)

        # this one should fail with a different error message
        self.client.post(self.auth_url, data={'username': self.username, 'password': 'WRONG'})
        self.assertEqual(response.data['detail'], 'Username is locked out. Try in 10 minutes.')
        self.assertEqual(response.status_code, status.HTTP_401_UNAUTHORIZED)

    def test_expired_token_is_recreated_on_successful_authentication(self):
        user = get_user_model().objects.get(username=self.username)
        self.assertIsNotNone(user.token_lifetime)
        response = self.client.post(self.auth_url, data={'username': self.username, 'password': self.password})
        self.assertEqual(response.status_code, status.HTTP_200_OK)
        token1 = response.data['token']

        mocked_now = timezone.now() + timezone.timedelta(seconds=user.token_lifetime)
        with freeze_time(mocked_now):
            response = self.client.post(self.auth_url, data={'username': self.username, 'password': self.password})
            token2 = response.data['token']
            self.assertNotEqual(token1, token2)

    def test_not_expired_token_creation_time_is_updated_on_authentication(self):
        response = self.client.post(self.auth_url, data={'username': self.username, 'password': self.password})
        self.assertEqual(response.status_code, status.HTTP_200_OK)
        token1 = response.data['token']
        created1 = Token.objects.values_list('created', flat=True).get(key=token1)

        response = self.client.post(self.auth_url, data={'username': self.username, 'password': self.password})
        token2 = response.data['token']
        created2 = Token.objects.values_list('created', flat=True).get(key=token2)

        self.assertEqual(token1, token2)
        self.assertTrue(created1 < created2)

    def test_token_never_expires_if_token_lifetime_is_none(self):
        user = get_user_model().objects.get(username=self.username)
        user.token_lifetime = None
        user.save()

        response = self.client.post(self.auth_url, data={'username': self.username, 'password': self.password})
        self.assertEqual(response.status_code, status.HTTP_200_OK)
        original_token = response.data['token']

        year_ahead = timezone.now() + timezone.timedelta(days=365)
        with freeze_time(year_ahead):
            response = self.client.post(self.auth_url, data={'username': self.username, 'password': self.password})
            token_in_a_year = response.data['token']
            self.assertEqual(original_token, token_in_a_year)

    def test_token_created_date_is_refreshed_even_if_token_lifetime_is_none(self):
        response = self.client.post(self.auth_url, data={'username': self.username, 'password': self.password})
        self.assertEqual(response.status_code, status.HTTP_200_OK)

        user = get_user_model().objects.get(username=self.username)
        original_token_lifetime = user.token_lifetime
        original_created_value = user.auth_token.created
        user.token_lifetime = None
        user.save()

        last_refresh_time = timezone.now() + timezone.timedelta(seconds=original_token_lifetime)
        with freeze_time(last_refresh_time):
            response = self.client.post(self.auth_url, data={'username': self.username, 'password': self.password})
            self.assertEqual(response.status_code, status.HTTP_200_OK)
            token = response.data['token']

        user.auth_token.refresh_from_db()
        self.assertTrue(user.auth_token.created > original_created_value)

        user.token_lifetime = original_token_lifetime
        user.save()
        with freeze_time(last_refresh_time):
            self.client.credentials(HTTP_AUTHORIZATION='Token ' + token)
            response = self.client.get(self.test_url)
            self.assertEqual(response.status_code, status.HTTP_200_OK)





# Tests from this module use structure and OpenStack models with their quotas
# to test quotas behaviour. Ideally we need to test quotas based on some abstract or
# test only models, but it is not really supported by Django.

from django.test import TestCase
from waldur_core.structure import models as structure_models
from waldur_core.structure.tests import factories as structure_factories


class QuotaFieldTest(TestCase):

    def setUp(self):
        self.customer_quotas_names = [f.name for f in structure_models.Customer.get_quotas_fields()]

    def test_quotas_initialization_on_object_creation(self):
        customer = structure_factories.CustomerFactory()

        for quota_name in self.customer_quotas_names:
            self.assertTrue(customer.quotas.filter(name=quota_name).exists(),
                            'Quota with name "%s" was not added to customer on creation' % quota_name)


class CounterQuotaFieldTest(TestCase):

    def test_target_model_instance_creation_increases_scope_counter_quota(self):
        customer = structure_factories.CustomerFactory()
        structure_factories.ProjectFactory(customer=customer)

        self.assertEqual(customer.quotas.get(name='nc_project_count').usage, 1)

    def test_target_model_instance_deletion_decreases_scope_counter_quota(self):
        customer = structure_factories.CustomerFactory()
        project = structure_factories.ProjectFactory(customer=customer)
        project.delete()

        self.assertEqual(customer.quotas.get(name='nc_project_count').usage, 0)



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('core', '0001_squashed_0003_ssh_key_name_length_changed'),
    ]

    operations = [
        migrations.AlterField(
            model_name='user',
            name='civil_number',
            field=models.CharField(null=True, default=None, max_length=50, blank=True, unique=True, verbose_name='civil number'),
        ),
    ]
